// CannonView.m
// Main view for the Cannon Game.
#import <AVFoundation/AVFoundation.h>
#import "CannonView.h"

static const int CANNON_BASE_RADIUS = 25; // radius of the cannon base
static const int CANNON_LENGTH = 40; // length of the cannon barrel

static const int CANNONBALL_RADIUS = 10; // radius of the cannonball
static const int CANNONBALL_VELOCITY = 600; // cannonball's velocity

static const int LINE_WIDTH = 10; // width of the target and blocker
static const int BLOCKER_DISTANCE = 200; // blocker distance from left
static const int BLOCKER_BEGINNING = 50; // blocker distance from top
static const int BLOCKER_END = 200; // blocker bottom's distance from top
static const int BLOCKER_VELOCITY = 300; // blocker's velocity

static const int TARGET_VELOCITY = -100; // target's velocity
static const int TARGET_DISTANCE = 300; // target's distance from left
static const int TARGET_BEGINNING = 50; // target's distance from top
static const int TARGET_END = 400; // target bottom's distance from top

static const int MISS_PENALTY = 2; // seconds subtracted for a miss
static const int HIT_REWARD = 3; // seconds added for a hit

static const float TIME_INTERVAL = 0.025; // interval for timer events

@implementation CannonView

// initialize the view
- (id)initWithCoder:(NSCoder *)aDecoder
{
   // if the superclass initialized properly
   if (self = [super initWithCoder:aDecoder])
   {
      // make the cannon initially point horizontally
      barrelEnd = CGPointMake(CANNON_LENGTH, self.frame.size.height / 2);
      
      // get the path for the cannon firing sound
      NSString *soundPath = [[NSBundle mainBundle]
         pathForResource:@"cannon_fire" ofType:@"wav"];
      
      // create a URL from the path
      NSURL *soundURL = [[NSURL alloc] initFileURLWithPath:soundPath];
      
      // initialize cannonFireSound with the URL
      cannonFireSound =
         [[AVAudioPlayer alloc] initWithContentsOfURL:soundURL error:nil];
      [soundURL release]; // release the soundURL NSURL
      
      // get the path for the target hit sound
      soundPath = [[NSBundle mainBundle] pathForResource:@"target_hit"
         ofType:@"wav"];
      
      // create a URL from the path
      soundURL = [[NSURL alloc] initFileURLWithPath:soundPath];
      
      // initialize targetHitSound with the URL
      targetHitSound =
         [[AVAudioPlayer alloc] initWithContentsOfURL:soundURL error:nil];
      [soundURL release]; // release the soundURL NSURL
      
      // get the path for the blocker hit sound
      soundPath = [[NSBundle mainBundle] pathForResource:@"blocker_hit"
                                                  ofType:@"wav"];
      
      // create a URL from the path
      soundURL = [[NSURL alloc] initFileURLWithPath:soundPath];
      
      // initialize blockerHitSound with the URL
      blockerHitSound =
         [[AVAudioPlayer alloc] initWithContentsOfURL:soundURL error:nil];
      [soundURL release]; // release the soundURL NSURL
   } // end if
   
   return self;
} // end method initWithCoder:

// called when the nib file finishes loading
- (void)awakeFromNib
{
   [self newGame]; // view is loaded, so start a new game
} // end method awakeFromNib

// reset all the screen elements and start a new game
- (void)newGame
{
   // set the initial blocker position based on the defined constants
   blocker.start = CGPointMake(BLOCKER_DISTANCE, BLOCKER_BEGINNING);
   blocker.end = CGPointMake(BLOCKER_DISTANCE, BLOCKER_END);
   
   // set the initial target position based on the defined constants
   target.start = CGPointMake(TARGET_DISTANCE, TARGET_BEGINNING);
   target.end = CGPointMake(TARGET_DISTANCE, TARGET_END);
   
   // set every element of targetPieceHit to NO
   for (int i = 0; i < TARGET_PIECES; i++)
      targetPieceHit[i] = NO; // this piece hasn't been hit

   targetPiecesHit = 0; // no target pieces have been hit
   
   blockerVelocity = BLOCKER_VELOCITY; // set the initial blocker velocity
   targetVelocity = TARGET_VELOCITY; // set the initial target velocity
   
   timeLeft = 10; // start the countdown at 10 seconds
   timerCount = 0; // the timer has fired 0 times so far
   cannonballOnScreen = NO; // the cannonball is not on the screen
   shotsFired = 0; // set initial number of shots fired
   timeElapsed = 0; // set the time elapsed to zero
   
   // start the timer with the defined time interval
   timer = [NSTimer scheduledTimerWithTimeInterval:TIME_INTERVAL target:
      self selector:@selector(timerFired:) userInfo:nil repeats:YES];
} // end method newGame

// refreshes all the elements on the screen
- (void)timerFired:(NSTimer *)theTimer
{
   // if the cannonball is on the screen
   if (cannonballOnScreen)
   {
      // update cannonball position
      cannonball.x += TIME_INTERVAL * cannonballVelocity.x;
      cannonball.y += TIME_INTERVAL * cannonballVelocity.y;
            
      // check for collision with blocker
      if (cannonball.x > BLOCKER_DISTANCE - CANNONBALL_RADIUS * 2 &&
          cannonball.x < BLOCKER_DISTANCE + LINE_WIDTH / 2 &&
          cannonball.y + CANNONBALL_RADIUS * 2 > blocker.start.y &&
          cannonball.y < blocker.end.y)
      {
         cannonballVelocity.x *= -1;
         [blockerHitSound play]; // play the blocker hit sound
         timeLeft -= MISS_PENALTY;
      } // end if
      
      // check for collisions with the left and right walls
      else if (cannonball.x > self.frame.size.width || cannonball.x +
         CANNONBALL_RADIUS * 2 < 0 && cannonballVelocity.x < 0)
         cannonballOnScreen = NO; // make the cannonball disappear
      
      // check for collisions with top and bottom walls
      else if (cannonball.y + CANNONBALL_RADIUS * 2 < 0 ||
         cannonball.y > self.frame.size.height + CANNONBALL_RADIUS * 2)
         cannonballOnScreen = NO; // make the cannonball disappear
      
      // check for a hit on the target
      else if (cannonball.x > TARGET_DISTANCE - CANNONBALL_RADIUS * 2 &&
         cannonball.x < TARGET_DISTANCE + LINE_WIDTH / 2 &&
         cannonball.y + CANNONBALL_RADIUS * 2 > target.start.y &&
         cannonball.y < target.end.y)
      {
         // calculate the length of each piece of the target
         float pieceLength =
            (TARGET_END - TARGET_BEGINNING) / TARGET_PIECES;
         
         // determine which section number was hit (0 is the top)
         int section =
            (int)((cannonball.y - target.start.y) / pieceLength);
         
         // check if the piece hasn't been hit yet
         if (!targetPieceHit[section])
         {
            targetPieceHit[section] = YES; // the piece was hit
            cannonballOnScreen = NO; // make the cannonball disappear
            timeLeft += HIT_REWARD; // add reward to remaining
            [targetHitSound play]; // play the target hit sound
            
            // if all the target's pieces have been hit
            if (++targetPiecesHit == TARGET_PIECES)
            {
               NSString *message = [NSString stringWithFormat:
                  @"Shots fired: %i\nTime elapsed: %i seconds",
                  shotsFired, timeElapsed];
               
               // display the game won alert
               [self showAlertWithTitle:@"You Win!" message:message];
            } // end if
         } // end if
      } // end else
   } // end if
   
   // update the blocker's position
   blocker.start.y += TIME_INTERVAL * blockerVelocity;
   blocker.end.y += TIME_INTERVAL * blockerVelocity;
   
   // update the target's position
   target.start.y += TIME_INTERVAL * targetVelocity;
   target.end.y += TIME_INTERVAL * targetVelocity;
   
   // check if the blocker has hit the top or bottom walls
   if (blocker.start.y < 0 || blocker.end.y > self.bounds.size.height)
      blockerVelocity *= -1; // reverse the blocker's direction
   
   // check if the target has hit the top or bottom walls
   if (target.start.y < 0 || target.end.y > self.bounds.size.height)
      targetVelocity *= -1; // reverse the target's direction
   
   ++timerCount; // increment the timer event counter
   
   // if one second has passed
   if (TIME_INTERVAL * timerCount >= 1)
   {
      --timeLeft; // subtract one from the timer
      ++timeElapsed; // increment the time elapsed
      timerCount = 0; // reset the count
   } // end if
   
   // if the timer reached zero
   if (timeLeft <= 0 && timerCount > 0)
      
      // display game over alert
      [self showAlertWithTitle:@"Game Over" message:nil];
   
   [self setNeedsDisplay]; // redraw the entire display
} // end method timerFired:

// show an alert with the given title and message
- (void)showAlertWithTitle:(NSString *)title message:(NSString *)message
{
   [timer invalidate]; // stop and release the timer
   timer = nil; // set timer to nil
   
   // create the game over alert
   UIAlertView *alert = [[UIAlertView alloc] initWithTitle:title
      message:message delegate:self cancelButtonTitle:@"New Game"
      otherButtonTitles:nil];
   [alert show]; // show the alert
   [alert release]; // release the alert UIAlertView
} // end method displayAlertWithTitle:

// called when the player touches the new game button in an alert
- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:
   (NSInteger)buttonIndex
{
   [self newGame]; // start a new game
} // end method alertView:clickedButtonAtIndex:

// draw the display in the given rectangle
- (void)drawRect:(CGRect)rect
{
   // get the current graphics context
   CGContextRef context = UIGraphicsGetCurrentContext();
   
   // save the context because we need to flip it upside down
   CGContextSaveGState(context);
   
   // translate the context down
   CGContextTranslateCTM(context, 0, self.bounds.size.height);
   CGContextScaleCTM(context, 1.0, -1.0); // flip context up over itself
   
   // create a string with the time remaining
   NSString *str =
      [NSString stringWithFormat:@"Time remaining: %i seconds", timeLeft];
   
   // make the font 16pt Helvetica
   CGContextSelectFont(context, "Helvetica", 16, kCGEncodingMacRoman);
   CGContextSetTextDrawingMode(context, kCGTextFill); // set drawing mode
   
   // set the text color to black
   CGContextSetRGBFillColor(context, 0.0, 0.0, 0.0, 1.0);
   
   // convert str to a C string and display it
   CGContextShowTextAtPoint(context, 10.0, 10.0,
      [str cStringUsingEncoding:[NSString defaultCStringEncoding]],
      str.length);
   CGContextRestoreGState(context); // restore the context
   
   // if the cannonball is on the screen
   if (cannonballOnScreen)
   {
      // create the rectangle to draw the cannonball in
      CGRect cannonballRect = CGRectMake(cannonball.x, cannonball.y,
         CANNONBALL_RADIUS * 2, CANNONBALL_RADIUS * 2);
      
      // load the cannonball image
      UIImage *image = [UIImage imageNamed:@"cannonball80.png"];
      
      // draw the image into the rectangle
      CGContextDrawImage(context, cannonballRect, image.CGImage);
   } // end if
   
   // draw the cannon barrel
   // move to the middle of the cannon base
   CGContextMoveToPoint(context, 0, self.frame.size.height / 2);
   
   // add a line to the endpoint of the cannon barrel
   CGContextAddLineToPoint(context, barrelEnd.x, barrelEnd.y);
   CGContextSetLineWidth(context, 20); // set line thickness
   CGContextSetRGBStrokeColor(context, 0.0, 0.0, 0.0, 1.0); // black color
   CGContextStrokePath(context); // draw the line
   
   // create the rectangle for the cannon base
   CGRect cannonBase = CGRectMake(0, self.frame.size.height / 2 -
      CANNON_BASE_RADIUS, CANNON_BASE_RADIUS, CANNON_BASE_RADIUS * 2);
   
   // load the image for the cannon base
   UIImage *baseImage = [UIImage imageNamed:@"cannon_base.png"];
   
   // draw the cannon base image into the rectangle
   CGContextDrawImage(context, cannonBase, baseImage.CGImage);
   
   // add a line between the two points of the blocker
   CGContextMoveToPoint(context, blocker.start.x, blocker.start.y);
   CGContextAddLineToPoint(context, blocker.end.x, blocker.end.y);
   CGContextSetLineWidth(context, LINE_WIDTH); // set the line width
   
   CGContextStrokePath(context); // draw the line
   
   // calculate the length of each piece in the target
   float pieceLength = (TARGET_END - TARGET_BEGINNING) / TARGET_PIECES;
   
   // move to the start point of the target
   CGContextMoveToPoint(context, target.start.x, target.start.y);
   
   // draw each target piece
   for (int i = 1; i <= TARGET_PIECES; i++)
   {
      // make the pieces different colors between yellow and blue
      if (i % 2 == 0)
         CGContextSetRGBStrokeColor(context, 1, 1, 0, 1);
      else
         CGContextSetRGBStrokeColor(context, 0, 0, 0.5, 1);
      
      // move to the ending point of the next segment
      CGContextMoveToPoint(context, target.end.x,
         target.start.y + pieceLength * (i - 1));
      
      // if the piece hasn't been hit yet
      if (!targetPieceHit[i - 1])
      {
         // add a line for the piece
         CGContextAddLineToPoint(context, target.end.x,
            target.start.y + pieceLength * i);
         CGContextStrokePath(context); // draw the piece
      } // end if
   } // end for
} // end method drawRect:

// handle new touches
- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event
{
   [self processTouch:[touches anyObject]];
} // end method touchesBegan:withEvent:

// exactly the same as touchesBegan:withEvent: minus double-tapping
- (void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event
{
   [self processTouch:[touches anyObject]];
} // end method touchesMoved:withEvent:

// aims the cannon and fires the cannonball if appropriate
- (void)processTouch:(UITouch *)touch
{
   // get the location of the touch in this view
   CGPoint touchPoint = [touch locationInView:self];
   
   // calculate the angle the barrel makes with the horizontal
   float angle =
      atan(touchPoint.x / (self.frame.size.height / 2 - touchPoint.y));
   
   // if the touch is on the lower half of the screen
   if (touchPoint.y > self.frame.size.height / 2)
      angle += M_PI; // adjust the angle
   
   // calculate the endpoint of the cannon barrel
   barrelEnd.x = CANNON_LENGTH * sin(angle);
   barrelEnd.y = -CANNON_LENGTH * cos(angle) + self.frame.size.height / 2;
   
   // update the display only in the area of the cannon
   [self setNeedsDisplayInRect:CGRectMake(0, self.frame.size.height / 2 -
      CANNON_BASE_RADIUS - CANNON_LENGTH, CANNON_BASE_RADIUS +
      CANNON_LENGTH, (CANNON_BASE_RADIUS + CANNON_LENGTH) * 2)];
   
   // fire a cannonball on a double-tap
   if (touch.tapCount >= 2 && !cannonballOnScreen)
   {
      // move the cannonball to be inside the cannon
      cannonball.x = -CANNONBALL_RADIUS; // align x-coordinate with cannon
      cannonball.y = self.frame.size.height / 2 - CANNONBALL_RADIUS; 
      
      // get the x component of the total velocity
      cannonballVelocity.x = CANNONBALL_VELOCITY * sin(angle);
      
      // get the y component of the total velocity
      cannonballVelocity.y = -CANNONBALL_VELOCITY * cos(angle);
      cannonballOnScreen = YES; // the cannonball is now visible
      ++shotsFired; // increment number of shots fired
      [cannonFireSound play]; // play the firing sound
   } // end if
} // end method processTouch:

// releases CannonView's memory
- (void)dealloc
{
   [cannonFireSound release]; // release the cannonFireSound AVAudioPlayer
   [targetHitSound release]; // release the targetHitSound AVAudioPlayer
   [blockerHitSound release]; // release the blockerHitSound AVAudioPlayer
   [super dealloc]; // invokes the superclass's dealloc method
} // end method dealloc
@end // end implementation of CannonView

/**************************************************************************
 * (C) Copyright 2010 by Deitel & Associates, Inc. All Rights Reserved.   *
 *                                                                        *
 * DISCLAIMER: The authors and publisher of this book have used their     *
 * best efforts in preparing the book. These efforts include the          *
 * development, research, and testing of the theories and programs        *
 * to determine their effectiveness. The authors and publisher make       *
 * no warranty of any kind, expressed or implied, with regard to these    *
 * programs or to the documentation contained in these books. The authors *
 * and publisher shall not be liable in any event for incidental or       *
 * consequential damages in connection with, or arising out of, the       *
 * furnishing, performance, or use of these programs.                     *
 *                                                                        *
 * As a user of the book, Deitel & Associates, Inc. grants you the        *
 * nonexclusive right to copy, distribute, display the code, and create   *
 * derivative apps based on the code for noncommercial purposes only--so  *
 * long as you attribute the code to Deitel & Associates, Inc. and        *
 * reference www.deitel.com/books/iPhoneFP/. If you have any questions,   *
 * or specifically would like to use our code for commercial purposes,    *
 * contact deitel@deitel.com.                                             *
 *************************************************************************/

