// SlideshowDataViewController.m
// SlideshowDataViewController class implementation.
#import "SlideshowDataViewController.h"
#import <MobileCoreServices/MobileCoreServices.h>

@implementation SlideshowDataViewController

// initialize with the given slideshow
- (id)initWithSlideshow:(Slideshow *)show
{
   // if the superclass initializes properly
   if (self = [super init])
      slideshow = [show retain]; // take ownership of the Slideshow
   
   return self; // return this object
} // end method initWithSlideshow

// method is called when view finishes initializing
- (void)viewDidLoad
{
   [super viewDidLoad]; // initialize the superclass
   [self.navigationItem setTitle:@"Edit Slideshow"]; // set the bar title
      
   // create the "Play" button
   // the "black translucent" style makes the button blue
   UIBarButtonItem *playButton = [[UIBarButtonItem alloc]
      initWithTitle:@"Play" style:UIBarStyleBlackTranslucent
      target:self action:@selector(startSlideshow)];
   
   // add the "Play" button to the top navigation bar on the right side
   [self.navigationItem setRightBarButtonItem:playButton];
   [playButton release]; // release the playButton UIBarButtonItem
   
   // create the toolbar at the bottom
   toolbar = [[UIToolbar alloc] init];
   [toolbar setBarStyle:UIBarStyleBlack]; // make toolbar black
   
   // create a "Add Picture" button for adding new photos
   UIBarButtonItem *pictureButton = [[UIBarButtonItem alloc]
      initWithTitle:@"Add Picture/Video" style:UIBarStyleBlack target:self
      action:@selector(addPhoto)];
   
   // create the "Add Music" button
   UIBarButtonItem *musicButton = [[UIBarButtonItem alloc]
      initWithTitle:@"Add Music" style:UIBarStyleBlack target:self
      action:@selector(addMusic)];
   
   // create the "Add Effect" button
   UIBarButtonItem *effectButton = [[UIBarButtonItem alloc]
      initWithTitle:@"Set Effect" style:UIBarStyleBlack target:self
      action:@selector(addEffect)];
   
   // create a flexible space bar button item
   UIBarButtonItem *space = [[UIBarButtonItem alloc]
      initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace
      target:nil action:NULL];
   
   // add the buttons to toolbar in the given order
   [toolbar setItems:[NSArray arrayWithObjects:space, pictureButton,
      effectButton, musicButton, space, nil]];
   
   [pictureButton release]; // release the pictureButton UIBarButtonItem
   [musicButton release]; // release the musicButton UIBarButtonItem
   [effectButton release]; // release the effectButton UIBarButtonItem
   
   // add the toolbar to the superview
   [[self.navigationController view] addSubview:toolbar];
   
   [toolbar sizeToFit]; // expand the toolbar to include all the buttons
   CGRect frame = toolbar.frame; // get the frame of toolbar
   frame.origin.y = self.navigationController.view.bounds.size.height -
      toolbar.bounds.size.height; // move toolbar to the bottom edge
   [toolbar setFrame:frame]; // apply the new frame
   
   // put the table's cells in editing mode so they display reorder and
   // delete controls
   [(UITableView *)self.view setEditing:YES];
   
   // set the height of each row
   [(UITableView *)self.view setRowHeight:rowHeight];
   
   firstLoad = YES; // initialize firstLoad
   returnFromImagePicker = NO; // initialize returnFromImagePicker
} // end method viewDidLoad

// method is called when view comes into view
- (void)viewDidAppear:(BOOL)animated
{
   [super viewDidAppear:animated]; // pass the message to the superclass
   
   // if this if the first time the view is appearing
   if (firstLoad)
   {
      // resize the table to fit the toolbar at the bottom
      firstLoad = NO;
      CGRect frame = [self.view frame]; // fetch the frame of view
      
      // decrease the height of the table
      frame.size.height -= [toolbar bounds].size.height;
      [self.view setFrame:frame]; // apply the new frame
   } // end if
   else
   {
      // unhide the toolbar and navigation bar
      CGRect frame = toolbar.frame; // fetch the frame of toolbar
      
      // set the frame to be just below the bottom of the screen
      frame.origin.y = self.navigationController.view.bounds.size.height;
      [toolbar setFrame:frame]; // apply the new frame
      [UIView beginAnimations:nil context:toolbar]; // begin animation
      frame.origin.y -= frame.size.height; // move the toolbar up
      [toolbar setFrame:frame]; // apply the new frame
      [UIView commitAnimations]; // end animation block
      
      // show the navigation bar at the top
      [self.navigationController setNavigationBarHidden:NO animated:YES];
   } // end else
} // end method viewDidAppear:

// called when the main view is going to disappear
- (void)viewWillDisappear:(BOOL)animated
{
   [super viewWillDisappear:animated]; // pass message to the superclass
   
   // hide the toolbar
   [UIView beginAnimations:nil context:toolbar]; // begin animation block
   CGRect frame = [toolbar frame]; // fetch the frame of toolbar
   frame.origin.y += frame.size.height; // move the toolbar down
   [toolbar setFrame:frame]; // apply the new frame
   [UIView commitAnimations]; // end animation block
} // end method viewWillDisappear:

// prompts the user to add a new photo or video to the slideshow
- (void)addPhoto
{
   // if imagePicker hasn't been initialized yet
   if (imagePicker == nil)
   {
      // create the image picker
      imagePicker = [[UIImagePickerController alloc] init];
      imagePicker.allowsEditing = YES; // allow user to edit images
      
      // pick images from the photo library
      imagePicker.sourceType =
         UIImagePickerControllerSourceTypePhotoLibrary;
      
      // allow all media types available in the photo library      
      imagePicker.mediaTypes =                                     
         [UIImagePickerController availableMediaTypesForSourceType:
         UIImagePickerControllerSourceTypePhotoLibrary];           
      imagePicker.delegate = self; // set delegate to this object
   } // end if
   
   returnFromImagePicker = YES; // we’re going to the image picker
   
   // show the image picker
   [self presentModalViewController:imagePicker animated:YES];
} // end method addPhoto

// called when the user finishes picking a photo or video
- (void)imagePickerController:(UIImagePickerController *)picker
   didFinishPickingMediaWithInfo:(NSDictionary *)info
{
   // make the image picker go away
   [self dismissModalViewControllerAnimated:NO];
   
   // if the user chose an image
   if ([[info valueForKey:@"UIImagePickerControllerMediaType"]
        isEqualToString:kUTTypeImage])                       
   {
      // get the image the user chose
      UIImage *image =
         [info objectForKey:UIImagePickerControllerEditedImage];
      
      // create a MediaItem with the image
      MediaItem *item = [[MediaItem alloc] initWithType:MediaTypeImage
         description:nil data:image];                                 
      
      // add the MediaItem to the slideshow
      [slideshow.media addObject:item];
      
      // insert the new image into the table
      UITableView *table = (UITableView *)self.view;
      
      // insert a new row in the table for the new picture
      [table insertRowsAtIndexPaths:[NSArray arrayWithObject:[NSIndexPath
         indexPathForRow:slideshow.media.count - 1 inSection:0]]
         withRowAnimation:UITableViewRowAnimationRight];
   } // end if
   else // if the user chose a video
   {
      // get the URL for the selected video
      NSURL *url = [info objectForKey:UIImagePickerControllerMediaURL];
      
      // get the name of the video
      NSString *name = [[url absoluteString] lastPathComponent];
      
      // find the location of this app's documents directory
      NSArray *paths = NSSearchPathForDirectoriesInDomains(
         NSDocumentDirectory, NSUserDomainMask, YES);
      
      // get the first directory
      NSString *directory = [paths objectAtIndex:0];
      
      // create a path in the documents directory for the video
      NSString *newPath = [directory stringByAppendingPathComponent:name];
      
      // get the default file manager
      NSFileManager *manager = [NSFileManager defaultManager];
      
      // move video from the temporary directory to documents directory
      [manager copyItemAtPath:[url path] toPath:newPath error:nil];
      
      // create a new MediaItemCreator
      MediaItemCreator *creator = [[MediaItemCreator alloc]
         initWithNibName:@"MediaItemCreator" bundle:nil];
      creator.delegate = self; // set delegate to this object
      
      // set the media to the chosen video
      creator.media = [NSURL fileURLWithPath:newPath];
      
      // show the MediaItemCreator
      [self presentModalViewController:creator animated:YES];
      [creator release]; // release the creator MediaItemCreator
   } // end else
} // end method imagePickerController:didFinishPickingMediaWithInfo:

// called when the user finishes naming the media
- (void)mediaItemCreator:(MediaItemCreator *)creator didCreateMediaItem:
   (MediaItem *)item;
{
   [slideshow.media addObject:item]; // add the media to the slideshow
   
   // add a new row in the table for the media
   UITableView *table = (UITableView *)self.view;
   
   // insert a new row in the table for the new picture
   [table insertRowsAtIndexPaths:[NSArray arrayWithObject:[NSIndexPath
      indexPathForRow:slideshow.media.count - 1 inSection:0]]
      withRowAnimation:UITableViewRowAnimationRight];
   
   // dismiss the MediaItemCreator
   [self dismissModalViewControllerAnimated:YES];
} // end method mediaItemCreator:didCreateMediaItem:

// called when the user touches the Add Music button
- (void)addMusic
{
   // create a new media picker configured for picking music
   musicPicker = [[MPMediaPickerController alloc] initWithMediaTypes:
      MPMediaTypeMusic];
   
   // allow the user to choose multiple songs
   musicPicker.allowsPickingMultipleItems = YES;
   musicPicker.delegate = self; // set delegate to this object
   
   // show the music picker
   [self presentModalViewController:musicPicker animated:YES];
   [musicPicker release]; // release musicPicker
} // end method addMusic

// called when the user touches the "Done" Button in the media picker
- (void)mediaPicker: (MPMediaPickerController *)mediaPicker
  didPickMediaItems:(MPMediaItemCollection *)mediaItemCollection
{
   // update the music of the slideshow
   slideshow.music = mediaItemCollection;
   
   // make the media picker go away
   [self dismissModalViewControllerAnimated:YES];
} // end method mediaPicker:didPickMediaItems:

// called when the user touches the "Add Effect" Button
- (void)addEffect
{
   // if effectSheet hasn't been created yet
   if (effectSheet == nil)
   {
      // create a new sheet with the given title and button titles
      effectSheet = [[UIActionSheet alloc] initWithTitle:@"Choose Effect"
         delegate:self cancelButtonTitle:nil destructiveButtonTitle:nil  
         otherButtonTitles:@"Fade", @"Slide", @"Flip", nil];    
   } // end if
   
   [effectSheet showInView:self.view]; // show the sheet
} // end method addEffect

// called when the user touches the Start Slideshow button
- (void)startSlideshow
{
   SlideshowViewController *controller = 
      [[SlideshowViewController alloc] init];
   
   // set the pictures to appear in the slideshow
   controller.slideshow = slideshow;
   
   // hide the navigation bar so the screen is clear for the slideshow
   [self.navigationController setNavigationBarHidden:YES animated:YES];
   
   // show the view that shows the slideshow
   [self.navigationController pushViewController:controller animated:YES];
   [controller release]; // release the controller SlideshowViewController
} // end method startSlideshow

// called when the user touches one of the options in the effect sheet
- (void)actionSheet:(UIActionSheet *)actionSheet clickedButtonAtIndex:
   (NSInteger)buttonIndex
{
   slideshow.effect = buttonIndex; // update the selected effect
} // end method actionSheet:clickedButtonAtIndex:

// called by the table view to determine the number of sections it has
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return 1; // the table only has one section
} // end method numberOfSectionsInTableView:
// called by the table view to get the number of rows in a given section
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:
   (NSInteger)section
{
   return slideshow.media.count; // return the number of media items
} // end method tableView:numberOfRowsInSection:

// called by the table view to get the cells it needs to populate itself
- (UITableViewCell *)tableView:(UITableView *)tableView
   cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
   static NSString *CellIdentifier = @"Cell";
   
   // get a reused cell
   UITableViewCell *cell =
      [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
   
   // if there were no cells that could be reused
   if (cell == nil)
      // create a new cell
      cell = [[[UITableViewCell alloc] initWithStyle:
         UITableViewCellStyleDefault reuseIdentifier:CellIdentifier]
         autorelease];
   
   // loop through all the existing subviews in the cell
   for (UIView *view in cell.contentView.subviews)
      [view removeFromSuperview]; // remove the view from the cell
   
   // get the MediaItem at the appropriate row for this cell
   MediaItem *item = [slideshow.media objectAtIndex:indexPath.row];
   
   // if the MediaItem represents an image
   if (item.type == MediaTypeImage)
   {
      // get the image for the given row
      UIImage *image = item.data;
      
      // create an image view for the image
      UIImageView *view = [[UIImageView alloc] initWithImage:image];
      
      // resize the image without distorting it
      float newWidth = image.size.width * rowHeight / image.size.height;
      CGRect frame;
      
      // create the image shifted to the right of the center by 50 points
      frame.origin.x = (cell.frame.size.width / 2) - (newWidth / 2) - 50;
      frame.origin.y = 0; // the image will fill the height of the cell
      frame.size.width = newWidth; // the width so there’s no distortion
      frame.size.height = rowHeight; // the image fills the entire height
      view.center = CGPointMake(cell.center.x - 50, cell.center.y);
      view.frame = frame; // assign the new frame
      [cell.contentView addSubview:view]; // add the image to the cell
      [view release]; // release the view UIImageView
   } // end if
   else  // if the MediaItem represents a video
   {
      UILabel *label = [[UILabel alloc] init]; // create a new label
      
      // set the label's text using the user's description of the video
      label.text = [NSString stringWithFormat:@"Video: %@", 
         item.description];
      CGRect frame = label.frame; // get the label's frame
      frame.origin.x = 20; // update the x-coordinate
      frame.size.height = rowHeight; // update the height
      frame.size.width = cell.contentView.frame.size.width; // set width
      label.frame = frame; // apply the new frame
      [cell.contentView addSubview:label]; // add the label to the cell
      [label release]; // release the label UILabel
   } // end else
   
   return cell; // return the configured cell
} // end method tableView:cellForRowAtIndexPath:

// called by the table view to determine the height of its rows
- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath
{
   return rowHeight; // all rows are the same height
} // end method tableView:heightForRowAtIndexPath:

// called by the table when the user touches the delete button
- (void)tableView:(UITableView *)tableView commitEditingStyle:
   (UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:
   (NSIndexPath *)indexPath
{
   // if the user touched a delete button
   if (editingStyle == UITableViewCellEditingStyleDelete)
   {
      // remove the object at the deleted row
      [slideshow.media removeObjectAtIndex:indexPath.row];
      
      // remove the row from the table
      [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:
         indexPath] withRowAnimation:UITableViewRowAnimationLeft];
   } // end if
} // end method tableView:commitEditingStyle:forRowAtIndexPath:

// used to reorder UITableViewCells
- (void)tableView:(UITableView *)tableView moveRowAtIndexPath:
   (NSIndexPath *)fromIndexPath toIndexPath:(NSIndexPath *)toIndexPath
{
   // get the image at the moved row
   MediaItem *item =
      [[slideshow.media objectAtIndex:fromIndexPath.row] retain];
   
   // remove the MediaItem
   [slideshow.media removeObjectAtIndex:fromIndexPath.row]; 
   
   // insert the image into the list at the specified index
   [slideshow.media insertObject:item atIndex:toIndexPath.row];
   [item release]; // release the item MediaItem
} // end method tableView:moveRowAtIndexPath:toIndexPath:

// called by the table view to check if a given row can be moved
- (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:
   (NSIndexPath *)indexPath
{
    return YES; // all the rows in this table can be moved
} // end method tableView:canMoveRowAtIndexPath:

// release this object's memory
- (void)dealloc
{
   [imagePicker release]; // release the imagePicker UIImagePicker
   [musicPicker release]; // release musicPicker MPMediaPickerController
   [toolbar release]; // release the toolbar UIToolbar
   [super dealloc]; // call the superclass's dealloc method
} // end method dealloc
@end

/**************************************************************************
 * (C) Copyright 2010 by Deitel & Associates, Inc. All Rights Reserved.   *
 *                                                                        *
 * DISCLAIMER: The authors and publisher of this book have used their     *
 * best efforts in preparing the book. These efforts include the          *
 * development, research, and testing of the theories and programs        *
 * to determine their effectiveness. The authors and publisher make       *
 * no warranty of any kind, expressed or implied, with regard to these    *
 * programs or to the documentation contained in these books. The authors *
 * and publisher shall not be liable in any event for incidental or       *
 * consequential damages in connection with, or arising out of, the       *
 * furnishing, performance, or use of these programs.                     *
 *                                                                        *
 * As a user of the book, Deitel & Associates, Inc. grants you the        *
 * nonexclusive right to copy, distribute, display the code, and create   *
 * derivative apps based on the code for noncommercial purposes only--so  *
 * long as you attribute the code to Deitel & Associates, Inc. and        *
 * reference www.deitel.com/books/iPhoneFP/. If you have any questions,   *
 * or specifically would like to use our code for commercial purposes,    *
 * contact deitel@deitel.com.                                             *
 *************************************************************************/

