//  SlideshowViewController.m
//  Controller for a view that shows a slideshow
#import "SlideshowViewController.h"

@implementation SlideshowViewController

@synthesize slideshow;

// set up the main view
- (void)loadView
{
   self.view = [[UIView alloc] init]; // initialize the main view
   
   // size the main view to fill the entire screen
   [self.view setFrame:CGRectMake(0, 0, 320, 480)];
} // end method loadView

// returns a UIImageView that contains the next image to display
- (UIImageView *)nextImageViewWithMedia:(MediaItem *)item
{
   // get the image at the next index
   UIImage *image = item.data;
   
   // create an image view for the image
   UIImageView *imageView = [[UIImageView alloc] initWithImage:image];
   
   CGRect screenBounds = self.view.bounds; // get the screen bounds
   
   // resize the image to fill the screen without distorting
   [imageView expandToFill:screenBounds];
   CGRect frame = imageView.frame; // get the frame of the image
   
   // position the image to appear in the center of the view
   frame.origin.x = (screenBounds.size.width - frame.size.width) / 2;
   frame.origin.y = (screenBounds.size.height - frame.size.height) / 2;
   imageView.frame = frame; // assign the new frame
   
   // Makes the image move proportionally in any direction if the
   // bounds of the superview change. Used during orientation changes.
   imageView.autoresizingMask = (UIViewAutoresizingFlexibleLeftMargin |
      UIViewAutoresizingFlexibleRightMargin |
      UIViewAutoresizingFlexibleTopMargin |
      UIViewAutoresizingFlexibleBottomMargin);
   
   return imageView; // return the configured image view
} // end method nextImageViewWithMedia:

// called from timerFired: if there are no more images to be displayed
- (void)exitShow
{
   [musicPlayer stop]; // stop the music
   
   // unhide the status bar
   [[UIApplication sharedApplication] setStatusBarHidden:NO];
   
   // remove the slideshow view to return to the edit slideshow view
   [self.navigationController popViewControllerAnimated:YES];
} // end method exitShow

// changes to the next slide in the slideshow
- (void)changeSlide
{
   // check if there’s another slide to display
   if (pictureIndex >= slideshow.media.count)
      [self exitShow]; // if there’s no image, exit the slideshow
   else
   {
      // get the next MediaItem
      MediaItem *item = [slideshow.media objectAtIndex:pictureIndex];
      
      // if the MediaItem represents an image
      if (item.type == MediaTypeImage)
         [self displayNewImage:item]; // display the image
      else // the MediaItem represents a video
         [self displayNewVideo:item]; // display the video
      
      ++pictureIndex; // increment the index
   } // end else
} // end method changeSlide

// displays a new image in the slideshow
- (void)displayNewImage:(MediaItem *)item
{
   // get the next image to display
   UIImageView *nextImageView = [self nextImageViewWithMedia:item];
   CGRect frame; // declare a new CGRect
   
   // transition to the image based on the transition effect
   switch (slideshow.effect)
   {
         // if the transition effect is fade
      case TransitionEffectFade:
         [self.view addSubview:nextImageView]; // add the image to view
         nextImageView.alpha = 0.0; // make the next image transparent
         
         // begin animation block
         [UIView beginAnimations:nil context:nextImageView];
         [UIView setAnimationDuration:2.0]; // set the animation length
         [UIView setAnimationDelegate:self]; // set the animation delegate
         
         // call the given method when the animation ends
         [UIView setAnimationDidStopSelector:
          @selector(transitionFinished:finished:context:)];
         
         [nextImageView setAlpha:1.0]; // fade in the next image
         [currentImageView setAlpha:0.0]; // fade out the old image
         
         [UIView commitAnimations]; // end animation block
         break;
         
         // if the transition effect is slide
      case TransitionEffectSlide:
         // position the next image to the right of the screen
         [self.view addSubview:nextImageView]; // add the image to view
         frame = nextImageView.frame;
         frame.origin.x += frame.size.width;
         nextImageView.frame = frame;
         
         // begin animation block
         [UIView beginAnimations:nil context:nextImageView];
         [UIView setAnimationDuration:2.0]; // set the animation length
         [UIView setAnimationDelegate:self]; // set the animation delegate
         
         // call the given method when the animation ends
         [UIView setAnimationDidStopSelector:
          @selector(transitionFinished:finished:context:)];
         
         frame.origin.x -= frame.size.width; // slide new image left
         nextImageView.frame = frame; // apply the new frame
         CGRect currentImageFrame = currentImageView.frame;
         
         // slide the old image to the left
         currentImageFrame.origin.x -= currentImageFrame.size.width;
         currentImageView.frame = currentImageFrame;
         
         [UIView commitAnimations]; // end animation block
         break;
      case TransitionEffectFlip: // if the transition effect is flip
         
         // begin the animation block
         [UIView beginAnimations:@"flip" context:nextImageView];
         [UIView setAnimationDuration:2.0]; // set the animation duration
         [UIView setAnimationDelegate:self]; // set delegate to self
         
         // call the given method when the animation finishes
         [UIView setAnimationDidStopSelector:
          @selector(transitionFinished:finished:context:)];
         
         // set the transition to flip from the right side
         [UIView setAnimationTransition:
          UIViewAnimationTransitionFlipFromRight forView:
          self.view cache:NO];
         
         [currentImageView removeFromSuperview]; // remove the last image
         [self.view addSubview:nextImageView]; // add the new image
         [UIView commitAnimations]; // start the animation
         break;
   } // end switch
   
   // change the slide after 5 seconds
   [self performSelector:@selector(changeSlide) withObject:nil
              afterDelay:5.0];
} // end method displayNewImage

// displays a new video in the slideshow
- (void)displayNewVideo:(MediaItem *)item
{
   // receive notifications when a movie player finishes playback
   [[NSNotificationCenter defaultCenter] addObserver:self
      selector:@selector(videoFinished:)
      name:MPMoviePlayerPlaybackDidFinishNotification object:nil];
   [moviePlayer release]; // release the old movie player
   
   // create a new movie player with the video URL
   moviePlayer =
      [[MPMoviePlayerController alloc] initWithContentURL:item.data];
   [moviePlayer play]; // play the video
} // end method displayNewVideo

// called when the video finishes playing
- (void)videoFinished:(NSNotification *)n
{
   // stop receiving notifications when videos finish
   [[NSNotificationCenter defaultCenter] removeObserver:self];
   
   // hide the status bar
   [[UIApplication sharedApplication] setStatusBarHidden:YES];
   [self changeSlide]; // change to the next slide
} // end method videoFinished

// call when the animation from the changing image finishes
- (void)transitionFinished:(NSString *)animationId finished:(BOOL)finished
   context:(void *)context
{
   // the the transition that finishes was flip or curl
   if (animationId != @"flip" && animationId != @"curl")
      [currentImageView removeFromSuperview]; // remove the old image
   
   [currentImageView release]; // release the memory for the old image
   currentImageView = context; // assign the new image
} // end method transitionFinished:finished:context:

// called when the view is going to come into view
- (void)viewWillAppear:(BOOL)animated
{
   [super viewWillAppear:animated]; // pass the message to the superclass
   pictureIndex = 0; // reset the index
   [self changeSlide];
   
   // hide the status bar so the slideshow can appear fullscreen
   [[UIApplication sharedApplication] setStatusBarHidden:YES];

   // if the user has selected music to play
   if (slideshow.music != nil)
   {
      // get the application music player
      musicPlayer = [MPMusicPlayerController applicationMusicPlayer];
      musicPlayer.shuffleMode = MPMusicShuffleModeOff; // turn off shuffle
      musicPlayer.repeatMode = MPMusicRepeatModeNone; // turn off repeat
      
      // add the music the user selected to the queue
      [musicPlayer setQueueWithItemCollection:slideshow.music];
      [musicPlayer play]; // play the music
   } // end if
} // end method viewWillAppear:

// called when the view is going to disapear
- (void)viewDidDisappear:(BOOL)animated
{
   [super viewDidDisappear:animated]; // call superclass's method
   
   // display the navigation bar
   [self.navigationController setNavigationBarHidden:NO animated:NO];
} // end method viewDidDisappear:

// determines whether the view rotates when the iPhone orientation changes
- (BOOL)shouldAutorotateToInterfaceOrientation:
   (UIInterfaceOrientation)interfaceOrientation
{
   return YES; // allow rotation to all interface orientations
} // end method shouldAutorotateToInterfaceOrientation;

// called when the iPhone orientation changes
- (void)willRotateToInterfaceOrientation:
   (UIInterfaceOrientation)interfaceOrientation duration:
   (NSTimeInterval)duration
{
   // create a CGRect with the view's hight and width flipped
   CGRect bounds = CGRectMake(0, 0, self.view.bounds.size.height,
      self.view.bounds.size.width);
   
   // make the current image resize to fill the flipped bounds
   [currentImageView expandToFill:bounds];
} // end method willRotateToInterfaceOrientation:

// release this object's memory
- (void)dealloc
{
   [currentImageView release]; // release the currentImageView UIImageView
   [moviePlayer release]; // release moviePlayer MPMoviePlayerController
   [super dealloc]; // call the superclass's dealloc method
} // end method dealloc
@end // end implementation SlideshowviewController

@implementation UIImageView (Scaling) // extensions to UIImageView

// scale the view to fill the given bounds without distorting
- (void)expandToFill:(CGRect)bounds
{
   UIImage *image = self.image; // get the image of this view
   CGRect frame = self.frame; // get the frame of this view
   
   // check if the image is bound by its height
   if (image.size.height / image.size.width >
       bounds.size.height / bounds.size.width)
   {
      // expand the new height to fill the entire view
      frame.size.height = bounds.size.height;
      
      // calculate the new width so the image isn't distorted
      frame.size.width = image.size.width * bounds.size.height /
         image.size.height;
      
      // add to the x and y coordinates so the view remains centered
      frame.origin.y += (self.frame.size.height - frame.size.height) / 2;
      frame.origin.x += (self.frame.size.width - frame.size.width) / 2;
   } // end if
   else // the image is bound by its width
   {
      // expand the new width to fill the entire view
      frame.size.width = bounds.size.width;
      
      // calculate the new height so the image isn't distorted
      frame.size.height = image.size.height * bounds.size.width /
         image.size.width;
      
      // add to the x and y coordinates so the view remains centered
      frame.origin.y += (self.frame.size.height - frame.size.height) / 2;
      frame.origin.x += (self.frame.size.width - frame.size.width) / 2;
   } // end else
   
   self.frame = frame; // assign the new framw
} // end method expandToFill:
@end // end implementation UIImageView (Scaling)

/**************************************************************************
 * (C) Copyright 2010 by Deitel & Associates, Inc. All Rights Reserved.   *
 *                                                                        *
 * DISCLAIMER: The authors and publisher of this book have used their     *
 * best efforts in preparing the book. These efforts include the          *
 * development, research, and testing of the theories and programs        *
 * to determine their effectiveness. The authors and publisher make       *
 * no warranty of any kind, expressed or implied, with regard to these    *
 * programs or to the documentation contained in these books. The authors *
 * and publisher shall not be liable in any event for incidental or       *
 * consequential damages in connection with, or arising out of, the       *
 * furnishing, performance, or use of these programs.                     *
 *                                                                        *
 * As a user of the book, Deitel & Associates, Inc. grants you the        *
 * nonexclusive right to copy, distribute, display the code, and create   *
 * derivative apps based on the code for noncommercial purposes only--so  *
 * long as you attribute the code to Deitel & Associates, Inc. and        *
 * reference www.deitel.com/books/iPhoneFP/. If you have any questions,   *
 * or specifically would like to use our code for commercial purposes,    *
 * contact deitel@deitel.com.                                             *
 *************************************************************************/

