//  Controller.m
//  Controller class for the Favorite Twitter Searches app.
#import "Controller.h" //this line is auto-generated

@implementation Controller //this line is auto-generated

// called when object is initialized
- (id)init
{
   self = [super init]; // initialize the superclass members
   
   if (self != nil) // if the superclass initialized properly
   {      
      // creates list of valid directories for saving a file
      NSArray *paths = NSSearchPathForDirectoriesInDomains(
         NSDocumentDirectory, NSUserDomainMask, YES);
      
      // get the first directory
      NSString *dir = [paths objectAtIndex:0];
      
      // concatenate the file name "tagsIndex.plist" to the path
      filePath = [[NSString alloc] initWithString:
         [dir stringByAppendingPathComponent:@"tagsIndex.plist"]];
      
      NSFileManager *fileManager = [NSFileManager defaultManager];
      
      // if the file does not exist, create an empty NSMutableDictionary;
      // otherwise, initialize an NSDictionary with the file's contents 
      if ([fileManager fileExistsAtPath:filePath] == NO)
      {
         tags = [[NSMutableDictionary alloc] init];
      } // end if
      else
      {
         tags = [[NSMutableDictionary alloc]
                 initWithContentsOfFile:filePath];
      } // end else
      
      buttons = [[NSMutableArray alloc] init]; // create array
      infoButtons = [[NSMutableArray alloc] init]; // create array
   } // end if
   
   return self;  // if self == nil, object not initialized properly
} // end method init

// called when the GUI components finish loading
- (void)awakeFromNib
{
   for (NSString *title in tags)
      [self addNewButtonWithTitle:title];
} // end method awakeFromNib

// remove all buttons and populate View with favorites
- (void)refreshList
{
   // remove all the buttons from the GUI
   for (UIButton *button in scrollView.subviews)
      [button removeFromSuperview];
      
   [infoButtons removeAllObjects];
   
   float buttonOffset = BUTTON_SPACING; // reset the spacing
   
   // repopulate the scroll view with buttons
   for (UIButton *button in buttons)
   {
      CGRect buttonFrame = button.frame; // fetch the frame of button
      buttonFrame.origin.x = BUTTON_SPACING; // set the x-coordinate
      buttonFrame.origin.y = buttonOffset; // set the y-coordinate
      
      // button width is the size of the view minus padding on each side
      buttonFrame.size.width = 
         scrollView.frame.size.width - 5 * BUTTON_SPACING;
      buttonFrame.size.height = BUTTON_HEIGHT; // set the height of button
      button.frame = buttonFrame; // assign the new frame to button
      [scrollView addSubview:button]; // add button as a subview
      
      // create detail button
      UIButton *infoButton = 
         [UIButton buttonWithType: UIButtonTypeDetailDisclosure];
      [infoButtons addObject:infoButton]; // add infoButton to infoButtons
      
      // position button to the right of the button we just added
      buttonFrame = infoButton.frame; // fetch the frame of infoButton
      buttonFrame.origin.x = scrollView.frame.size.width - 35;
      
      // this button is a bit shorter than normal buttons, so we adjust
      buttonFrame.origin.y = buttonOffset+3;
      infoButton.frame = buttonFrame; // assign the new frame
      
      // make the button call infoButtonTouched: when it is touched
      [infoButton addTarget:self action:@selector(infoButtonTouched:)
           forControlEvents:UIControlEventTouchUpInside];
      [scrollView addSubview:infoButton]; // add infoButton as a subview
      
      // increase the offset so the next button is added further down
      buttonOffset += BUTTON_HEIGHT + BUTTON_SPACING;
   } // end for
} // end refreshList

// called when the user touches an info button
- (void)infoButtonTouched:sender
{
   // get the index of the button that was touched
   int index = [infoButtons indexOfObject:sender];
   
   // get the title of the button
   NSString *key = [[buttons objectAtIndex:index] titleLabel].text;
   tagField.text = key; // update tagField with the button title
   
   // get the search query using the button title
   NSString *value = [tags valueForKey:key];
   queryField.text = value; // update queryField with the value
} // end method infoButtonTouched:

// add a favorite search
- (IBAction)addTag:sender
{
   // make the keyboard disappear
   [tagField resignFirstResponder];
   [queryField resignFirstResponder];
   
   NSString *key = tagField.text; // get the text in tagField
   NSString *value = queryField.text; // get the text in queryField
   
   // test if either field is empty
   if (value.length == nil || key.length == nil)
      return; // exit from the method
   
   if ([tags valueForKey:key] == nil) // test if the tag already exists
      [self addNewButtonWithTitle:key]; // if not, add a new button
      
   [tags setValue:value forKey:key]; // add a new entry in tags
   
   tagField.text = nil; // clear tagField of text
   queryField.text = nil; // clear queryField of text
   
   [tags writeToFile:filePath atomically:NO]; //save the data
} // end method addTag:

// remove all the tags
- (IBAction)clearTags:sender
{
   [tags removeAllObjects]; // clear tags
   [tags writeToFile:filePath atomically:NO]; // update favorite file
   [buttons removeAllObjects]; // clear buttons
   [self refreshList]; // update the display
} // end clearTags:

// add a new button with the given title to the bottom of the list
- (void)addNewButtonWithTitle:(NSString *)title
{
   // create a new button
   UIButton *button = [UIButton buttonWithType:UIButtonTypeRoundedRect];
   
   // give the button the title of the tag
   [button setTitle:title forState:UIControlStateNormal];
   
   // tell the button to call buttonTouched: when it is touched
   [button addTarget:self action:@selector(buttonTouched:)
    forControlEvents:UIControlEventTouchUpInside];
   
   [buttons addObject:button]; // add the UIButton to the end of the array
   
   // sort the NSMutableArray by the UIButton's titles
   [buttons sortUsingSelector:@selector(compareButtonTitles:)];
   
   [self refreshList]; // refresh the list of favorite search Buttons
   
   // Adjust the content size of the view to include the new button. The
   // view scrolls only when the content size is greater than its frame.
   CGSize contentSize = CGSizeMake(scrollView.frame.size.width,
      buttons.count * (BUTTON_HEIGHT + BUTTON_SPACING) + BUTTON_SPACING);
   [scrollView setContentSize:contentSize];
} // end addNewButtonWithTitle:

// load selected search in web browser
- (void)buttonTouched:sender
{
   NSString *key = [sender titleLabel].text; // get Button's text       
   
   // get the search and replace special characters with percent escapes
   NSString *search = [[tags valueForKey:key]
      stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding];
   
   // format the URL
   NSString *urlString = [NSString stringWithFormat:
      @"http://search.twitter.com/search?q=%@", search];
   NSURL *url = [NSURL URLWithString:urlString];
   [[UIApplication sharedApplication] openURL:url];
} // end buttonTouched
@end // end implementation Controller

// define UIButton's sorting category method
@implementation UIButton (sorting)
- (NSComparisonResult)compareButtonTitles:(UIButton *)button
{
   // compare this UIButton's title to that of the given UIButton
   return [self.titleLabel.text 
      caseInsensitiveCompare:button.titleLabel.text];
} // end method compareButtonTitles
@end // end UIButtons's sorting category


/**************************************************************************
 * (C) Copyright 2010 by Deitel & Associates, Inc. All Rights Reserved.   *
 *                                                                        *
 * DISCLAIMER: The authors and publisher of this book have used their     *
 * best efforts in preparing the book. These efforts include the          *
 * development, research, and testing of the theories and programs        *
 * to determine their effectiveness. The authors and publisher make       *
 * no warranty of any kind, expressed or implied, with regard to these    *
 * programs or to the documentation contained in these books. The authors *
 * and publisher shall not be liable in any event for incidental or       *
 * consequential damages in connection with, or arising out of, the       *
 * furnishing, performance, or use of these programs.                     *
 *                                                                        *
 * As a user of the book, Deitel & Associates, Inc. grants you the        *
 * nonexclusive right to copy, distribute, display the code, and create   *
 * derivative apps based on the code for noncommercial purposes only--so  *
 * long as you attribute the code to Deitel & Associates, Inc. and        *
 * reference www.deitel.com/books/iPhoneFP/. If you have any questions,   *
 * or specifically would like to use our code for commercial purposes,    *
 * contact deitel@deitel.com.                                             *
 *************************************************************************/

