// MainViewController.m
// Controller for the front side of the Flag Quiz app.
#import <AVFoundation/AVFoundation.h>
#import <MediaPlayer/MediaPlayer.h> 
#import "MainViewController.h"
#import "MainView.h"

static const int BAR_OFFSET = 247; // top Segmented Control's y-coordinate

@implementation MainViewController

// generate get and set methods for outlet properties
@synthesize flagView; 
@synthesize answerLabel; 
@synthesize numCorrectLabel;   

// initialize the controller
- (id)initWithNibName:(NSString *)nibNameOrNil
   bundle:(NSBundle *)nibBundleOrNil
{
   // initialize the superclass
   if (self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil])
   {
      guessRows = 1; // default to one row of choices
      bars = [[NSMutableArray alloc] init]; // initialize the list of bars
      filenames = [[NSMutableArray alloc] init];
      
      // initialize the list of flags to be displayed
      quizCountries = [[NSMutableArray alloc] initWithCapacity:10];
      
      // create the dictionary of regions
      regions = [[NSMutableDictionary alloc] init];
      
      // default all the regions to on 
      NSNumber *yesBool = [NSNumber numberWithBool:YES];
      [regions setValue:yesBool forKey:@"Africa"];
      [regions setValue:yesBool forKey:@"Asia"];
      [regions setValue:yesBool forKey:@"Europe"];
      [regions setValue:yesBool forKey:@"North_America"];
      [regions setValue:yesBool forKey:@"Oceania"];
      [regions setValue:yesBool forKey:@"South_America"];
   } // end if
   
   return self; // return this MainViewController
} // end method initWithNibName:bundle:

// called when the view finishes loading
- (void)viewDidLoad
{
   [super viewDidLoad]; // call superclass's viewDidLoad method
   srandom(time(0)); // seed random number generator
   
   [answerLabel setText:nil]; // clear answerLabel
   [numCorrectLabel setText:@"Question 1 of 10"]; // initialize label
   
   // get a list of all the png files in the app
   NSMutableArray *paths = [[[NSBundle mainBundle]
      pathsForResourcesOfType:@"png" inDirectory:nil] mutableCopy];
   
   // loop through each png file
   for (NSString *filename in paths)
   {
      // separate the file name from the rest of the path
      filename = [filename lastPathComponent];
      [filenames addObject:filename]; // add the display name
   } // end for
   
   [paths release]; // release the paths NSMutableArray
   [self resetQuiz]; // start a new quiz
} // end viewDidLoad

// called 3 seconds after the user guesses a correct flag
- (void)loadNextFlag
{
   // get file name of the next flag
   NSString *nextImageName = [[quizCountries lastObject] retain];
   [quizCountries removeLastObject]; // remove that flag from list
   correctAnswer = nextImageName; // update the correct answer
   
   // create a new flag image using the given file name
   UIImage *nextImage = [UIImage imageNamed:nextImageName];
   
   // create a UIImageView for the next flag
   UIImageView *nextImageView =
      [[UIImageView alloc] initWithImage:nextImage];
   
   // delete the current flagView and put nextImageView in its place
   [nextImageView setFrame:[flagView frame]]; // copy the frame over
   [flagView removeFromSuperview]; // remove flagView from view
   [flagView release]; // release the flagView's memory
   flagView = nextImageView; // reassign flagView to the new view
   [self.view addSubview:flagView]; // add the new view to view
   
   int offset = BAR_OFFSET + 40 * bars.count; // set offset for next bar
   
   // add new UISegmentedControls if necessary
   for (int i = bars.count; i < guessRows; i++)
   {
      // create a new bar with three empty items
      UISegmentedControl *bar = [[UISegmentedControl alloc] initWithItems:
         [NSArray arrayWithObjects:@"", @"", @"", nil]];
      bar.segmentedControlStyle = UISegmentedControlStyleBar;
      
      // make the segments stay selected only momentarily
      bar.momentary = YES;
      
      // tell the bar to call the given method whenever it's touched
      [bar addTarget:self action:@selector(submitGuess:)
         forControlEvents:UIControlEventValueChanged];
      CGRect frame = bar.frame; // get the current frame for the bar
      frame.origin.y = offset; // position it below the last bar
      frame.origin.x = 20; // give it some padding on the left
      
      // expand the bar to fill the screen with some padding on the right
      frame.size.width = self.view.frame.size.width - 40;
      bar.frame = frame; // assign the new frame
      [self.view addSubview:bar]; // add the bar to the main view
      [bars addObject:bar]; // add the bar to the list of bars
      [bar release]; // release the bar Segmented Control
      offset += 40; // increase the offset so the next bar is farther down
   } // end for
   
   // delete bars if there are too many on the screen
   for (int i = bars.count; i > guessRows; i--)
   {
      UISegmentedControl *bar = [bars lastObject]; // get the last bar
      [bar removeFromSuperview]; // remove the bar from the main view
      [bars removeLastObject]; // remove the bar from the list of bars
   } // end for
   
   // enable all the bars
   for (UISegmentedControl *bar in bars)
   {
      bar.enabled = YES; // enable the Segmented Control
      
      // enable each segment of the bar
      for (int i = 0; i < 3; i++)
         [bar setEnabled:YES forSegmentAtIndex:i];
   } // end for
   
   // shuffle filenames
   for (int i = 0; i < filenames.count; i++)
   {
      // pick a random int between the current index and the end
      int n = (random() % (filenames.count - i)) + i;
      
      // swap the object at index i with the index randomly picked
      [filenames exchangeObjectAtIndex:i withObjectAtIndex:n];
   } // end for
   
   // get the index of the string with the correct answer
   int correct = [filenames indexOfObject:correctAnswer];
   
   // put the correct answer at the end
   [filenames exchangeObjectAtIndex:filenames.count - 1
      withObjectAtIndex:correct];
   
   int flagIndex = 0; // start adding flags from the beginning
   
   // loop through each bar and choose incorrect answers to display
   for (int i = 0; i < guessRows; i++)
   {
      // get the bar at the current index
      UISegmentedControl *bar =
         (UISegmentedControl *)[bars objectAtIndex:i];
      int segmentIndex = 0;
      
      // loop through each segment of the bar
      while (segmentIndex < 3)
      {
         NSString *name; // store country name
         
         // if there is another filename
         if (flagIndex < filenames.count)
            name = [filenames objectAtIndex:flagIndex]; // get filename
         else // there aren't enough names to display
            name = nil; // set name to nil
         
         // get the region from the file name
         NSString *region =
            [[name componentsSeparatedByString:@"-"] objectAtIndex:0];
         
         // if the region of the selected country is enabled
         if ([[regions valueForKey:region] boolValue])
         {
            [bar setTitle:[name convertToDisplayName]
               forSegmentAtIndex:segmentIndex];
            ++segmentIndex;
         } // end if
         
         ++flagIndex; // move to the next entry in the array
      } // end while
   } // end for
   
   int z = random() % guessRows; // pick a random bar
   UISegmentedControl *bar = [bars objectAtIndex:z];
   
   // put the correct answer on a randomly chosen segment
   [bar setTitle:[correctAnswer convertToDisplayName]
      forSegmentAtIndex:random() % 3];
   
   // update the label to display the current question number
   [numCorrectLabel setText:[NSString stringWithFormat:
      @"Question %i of 10", numCorrect + 1]];
   [answerLabel setText:nil]; // clear the answer label
} // end method loadNextFlag

// called when the user touches one of the Segmented Control items
- (IBAction)submitGuess:sender
{
   // get the index of the selected item
   int index = [sender selectedSegmentIndex];
   
   // get the title of the bar at that segment, which is the guess
   NSString *guess = [sender titleForSegmentAtIndex:index];
   ++totalGuesses; // increment the number of times the user has guessed
   
   // if the guess is correct
   if ([guess isEqualToString:[correctAnswer convertToDisplayName]])
   {
      // make the text color a medium green
      answerLabel.textColor =
         [UIColor colorWithRed:0.0 green:0.7 blue:0.0 alpha:1.0];
      answerLabel.text = @"Correct!"; // set the text in the label
      
      // get the correct answer from the correct file name
      NSString *correct = [correctAnswer convertToDisplayName];
      
      // loop through each bar
      for (UISegmentedControl *bar in bars)
      {
         bar.enabled = NO; // don't let the user choose another answer
         
         // loop through the bar segments
         for (int i = 0; i < 3; i++)
         {
            // get the segment's title
            NSString *title = [bar titleForSegmentAtIndex:i];
            
            // if this segment does not have the correct choice
            if (![title isEqualToString:correct])
               [bar setEnabled:NO forSegmentAtIndex:i]; // disable segment
         } // end for
      } // end for
      
      ++numCorrect; // increment the number of correct answers
      
      // is the game finished?
      if (numCorrect == 10)
      {
         // create the message which includes guess number and percentage
         NSString *message = [NSString stringWithFormat:
            @"%i guesses, %.02f%% correct", totalGuesses,
            1000 / (float)totalGuesses];
         
         // create an alert to display the message
         UIAlertView *alert = [[UIAlertView alloc] initWithTitle:
            @"Results" message:message delegate:self cancelButtonTitle:
            @"Reset Quiz" otherButtonTitles:nil];
         [alert show]; // show the alert
         [alert release]; // release the alert UIAlertView
      } // end if
      else // the game is not finished so load another flag
      {
         // load a new flag after 3 seconds
         [self performSelector:@selector(loadNextFlag) withObject:nil
            afterDelay:3];
      } // end else
   } // end if
   else // the user has guessed incorrectly
   {
      answerLabel.textColor = [UIColor redColor]; // set text color to red
      answerLabel.text = @"Incorrect"; // set the text of the label
      
      // disable the incorrect choice
      [sender setEnabled:NO forSegmentAtIndex:index];
   } // end else
} // end method submitGuess:

// called when the user touches the "Reset Quiz" button in the alert
- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:
   (NSInteger)buttonIndex
{
   [self resetQuiz]; // reset the quiz
} // end method alertView:clickedButtonAtIndex:

// set the number of bars for displaying choices
- (void)setGuessRows:(int)rows
{
   guessRows = rows;
} // end method setGuessRows:

// reset the quiz
- (void)resetQuiz
{
   numCorrect = 0; // reset the number of correct answers the user made
   totalGuesses = 0; // reset the total number of guesses the user made
   int i = 0; // initialize i to 0
   
   // add 10 random file names to quizCountries
   while (i < 10)
   {
      int n = random() % filenames.count; // choose a random index
      
      // get the filename from the end of the path
      NSString *filename = [filenames objectAtIndex:n];
      NSArray *components = [filename componentsSeparatedByString:@"-"];
      
      // get the region from the beginning of the filename
      NSString *region = [components objectAtIndex:0];
      
      // check if the region is enabled
      NSNumber *regionEnabled = [regions valueForKey:region];
      
      // if the region is enabled and it hasn't already been chosen
      if ([regionEnabled boolValue] &&
          ![quizCountries containsObject:filename])
      {
         [quizCountries addObject:filename]; // add the file to the list
         ++i; // increment i
      } // end if
   } // end for
   
   [self loadNextFlag]; // load the first flag
} // end method resetQuiz:

// returns the NSMutableDictionary regions
- (NSMutableDictionary *)regions
{
   return regions; // return the regions NSMutableDictionary
} // end method regions

// called by a FlipsideViewController when the user touches "Done"
- (void)flipsideViewControllerDidFinish:
   (FlipsideViewController *)controller
{
   // flip the app back to the front side
   [self dismissModalViewControllerAnimated:YES];
} // end method flipsideViewControllerDidFinish:

// called when the user touches the info button
- (IBAction)showInfo
{
   // create a new FlipsideViewController
   FlipsideViewController *controller = [[FlipsideViewController alloc]
      initWithNibName:@"FlipsideView" bundle:nil];
   controller.delegate = self; // set the delegate
   
   // set the animation style to a horizontal flip
   controller.modalTransitionStyle = UIModalTransitionStyleFlipHorizontal;
   
   // show the flipside of the app
   [self presentModalViewController:controller animated:YES];
   
   // set the controls on the flipside
   [controller setSwitches:regions]; // set each region's switch
   [controller setSelectedIndex:guessRows - 1]; // set number of choices

   [controller release]; // release the controller FlipsideViewController
} // end method showInfo

- (void)dealloc
{
   [bars release]; // release the bars NSMutableArray
   [filenames release]; // release the filenames NSMutableArray
   [quizCountries release]; // release quizCountries NSMutableDictionary
   [flagView release]; // release the flagView UIImageView
   [answerLabel release]; // release the answerLabel UILabel
   [numCorrectLabel release]; // release the numCorrectLabel UILabel
   [super dealloc]; // release the superclass
} // end method dealloc
@end // end implementation of MainViewController

@implementation NSString (displayName)
- (NSString *)convertToDisplayName
{
   // get the name from the end of the string after the hyphen
   NSString *name = [[self componentsSeparatedByString:@"-"]
      objectAtIndex:1];
   
   // get a mutable copy of the name for editing
   NSMutableString *displayName = [[name mutableCopy] autorelease];
   
   // remove the .png from the end of the name
   [displayName replaceOccurrencesOfString:@".png" withString:@""
      options:NSLiteralSearch range:NSMakeRange(0, displayName.length)];
   
   // replace all underscores with spaces
   [displayName replaceOccurrencesOfString:@"_" withString:@" "
      options:NSLiteralSearch range:NSMakeRange(0, displayName.length)];
   
   return displayName;
} // end method convertToDisplayName
@end // end implementation of NSString


/**************************************************************************
 * (C) Copyright 2010 by Deitel & Associates, Inc. All Rights Reserved.   *
 *                                                                        *
 * DISCLAIMER: The authors and publisher of this book have used their     *
 * best efforts in preparing the book. These efforts include the          *
 * development, research, and testing of the theories and programs        *
 * to determine their effectiveness. The authors and publisher make       *
 * no warranty of any kind, expressed or implied, with regard to these    *
 * programs or to the documentation contained in these books. The authors *
 * and publisher shall not be liable in any event for incidental or       *
 * consequential damages in connection with, or arising out of, the       *
 * furnishing, performance, or use of these programs.                     *
 *                                                                        *
 * As a user of the book, Deitel & Associates, Inc. grants you the        *
 * nonexclusive right to copy, distribute, display the code, and create   *
 * derivative apps based on the code for noncommercial purposes only--so  *
 * long as you attribute the code to Deitel & Associates, Inc. and        *
 * reference www.deitel.com/books/iPhoneFP/. If you have any questions,   *
 * or specifically would like to use our code for commercial purposes,    *
 * contact deitel@deitel.com.                                             *
 *************************************************************************/

