// Controller.m
// Controller class for the Route Tracker app.
#import "Controller.h"

@implementation Controller

// generate get and set methods for our properties
@synthesize startButton;
@synthesize mapView;

// called when the main view finishes loading
- (void)viewDidLoad
{
   [super viewDidLoad]; // initialize the superclass
   tracking = NO; // set tracking to NO
   
   // initialize the TrackingMapView
   trackingMapView = 
      [[TrackingMapView alloc] initWithFrame:mapView.frame];
   
   // add the trackingMapView subview to mapView   
   [mapView addSubview:trackingMapView];
   [trackingMapView release]; // release the TrackingMapView
   
   // set trackingMapView as mapView's delegate
   mapView.delegate = trackingMapView; 
   
   // initialize location manager   
   locationManager = [[CLLocationManager alloc] init];
   
   // set locationManager's delegate to self
   locationManager.delegate = self; 
   
   // set locationManager to provide the most accurate readings possible
   locationManager.desiredAccuracy = kCLLocationAccuracyBest;
} // end method viewDidLoad

// called when the user touches the "Start Tracking" button
- (IBAction)toggleTracking
{
   // if the app is currently tracking
   if (tracking) 
   {
      tracking = NO; // stop tracking
      
      // allow the iPhone to go to sleep
      [[UIApplication sharedApplication] setIdleTimerDisabled:NO];
      startButton.title = @"Start Tracking"; // update button title
      [locationManager stopUpdatingLocation]; // stop tracking location
      [locationManager stopUpdatingHeading]; // stop tracking heading
      mapView.scrollEnabled = YES; // allow the user to scroll the map
      mapView.zoomEnabled = YES; // allow the user to zoom the map
      
      // get the time elapsed since the tracking started
      float time = -[startDate timeIntervalSinceNow];
      
      // format the ending message with various calculations
      NSString *message = [NSString stringWithFormat:
         @"Distance: %.02f km, %.02f mi\nSpeed: %.02f km/h, %.02f mph",
         distance / 1000, distance * 0.00062, distance * 3.6 / time, 
         distance * 2.2369 / time];
      
      // create an alert that shows the message
      UIAlertView *alert = [[UIAlertView alloc] 
         initWithTitle:@"Statistics" message:message delegate:self 
         cancelButtonTitle:@"Return" otherButtonTitles:nil];
      [alert show]; // show the alert
      [alert release]; // release the alert UIAlertView
   } // end if
   else // start tracking
   {
      tracking = YES; // start tracking
      mapView.scrollEnabled = NO; // prevent map scrolling by user 
      mapView.zoomEnabled = NO; // prevent map zooming by user
      
      // keep the iPhone from going to sleep
      [[UIApplication sharedApplication] setIdleTimerDisabled:YES];
      startButton.title = @"Stop Tracking"; // update button title
      distance = 0.0; // reset the distance
      startDate = [[NSDate date] retain]; // store the start time
      [locationManager startUpdatingLocation]; // start tracking location
      [locationManager startUpdatingHeading]; // start tracking heading  
   } // end else
} // end method toggleTracking

// called when the user touches the Reset button
- (IBAction)resetMap
{
   [trackingMapView reset]; // clear all the stored points
} // end method resetMap

// called when the user touches a segment of the UISegmentedControl
- (IBAction)selectMapMode:sender
{
   int index = [sender selectedSegmentIndex]; // get the selected segment
   
   // set the map type depending on the selected segment
   switch (index) 
   {
      // show the standard map
      case 0: mapView.mapType = MKMapTypeStandard; break;
         
      // show the satellite map
      case 1: mapView.mapType = MKMapTypeSatellite; break;
         
      // show the hybrid map
      case 2: mapView.mapType = MKMapTypeHybrid; break;
   } // end switch
} // end method selectMapMode:

// delegate method for the MKMapView
- (MKAnnotationView *)mapView:(MKMapView *)mapView viewForAnnotation:
   (id <MKAnnotation>)annotation
{
   return nil; // we don't want any annotations
} // end method mapView:viewForAnnotation:

// called whenever the location manager updates the current location
- (void)locationManager:(CLLocationManager *)manager didUpdateToLocation:
   (CLLocation *)newLocation fromLocation:(CLLocation *)oldLocation
{
   // add the new location to the map
   [trackingMapView addPoint:newLocation];
   
   // if there was a previous location
   if (oldLocation != nil)  
   {
      // add distance from the old location to the total distance
      distance += [newLocation getDistanceFrom:oldLocation];
   } 
   
   // create a region centered around the new point
   MKCoordinateSpan span = MKCoordinateSpanMake(0.005, 0.005);
   
   // create a new MKCoordinateRegion centered around the new location
   MKCoordinateRegion region =
      MKCoordinateRegionMake(newLocation.coordinate, span);
   
   // reposition the map to show the new point
   [mapView setRegion:region animated:YES];   
} // end method locationManager:didUpdateToLocation:fromLocation

// called when the location manager updates the heading
- (void)locationManager:(CLLocationManager *)manager didUpdateHeading:
   (CLHeading *)newHeading
{
   // calculate the rotation in radians
   float rotation = newHeading.trueHeading * M_PI / 180;
   
   // reset the transform
   mapView.transform = CGAffineTransformIdentity;
   
   // create a new transform with the angle
   CGAffineTransform transform = CGAffineTransformMakeRotation(-rotation);
   mapView.transform = transform; // apply the new transform
} // end method locationManager:didUpdateHeading:

// Write an error to console if the CLLocationManager fails
- (void)locationManager:(CLLocationManager *)manager didFailWithError:
(NSError *)error
{
   // stop using the location service if user disallowed access to it
   if ([error code] == kCLErrorDenied)
      [locationManager stopUpdatingLocation];
   
   NSLog(@"location manager failed"); // log location manager error
} // end method locationManager:didFailWithError:

// free Controller's memory
- (void)dealloc
{
   [startButton release]; // release the startButton UIBarButtonItem
   [mapView release]; // release the mapView MKMapView
   [locationManager release]; // release the CLLocationManager
   [super dealloc]; // call the superclass's dealloc
} // end method dealloc
@end // end Controller implementation

/**************************************************************************
 * (C) Copyright 2010 by Deitel & Associates, Inc. All Rights Reserved.   *
 *                                                                        *
 * DISCLAIMER: The authors and publisher of this book have used their     *
 * best efforts in preparing the book. These efforts include the          *
 * development, research, and testing of the theories and programs        *
 * to determine their effectiveness. The authors and publisher make       *
 * no warranty of any kind, expressed or implied, with regard to these    *
 * programs or to the documentation contained in these books. The authors *
 * and publisher shall not be liable in any event for incidental or       *
 * consequential damages in connection with, or arising out of, the       *
 * furnishing, performance, or use of these programs.                     *
 *                                                                        *
 * As a user of the book, Deitel & Associates, Inc. grants you the        *
 * nonexclusive right to copy, distribute, display the code, and create   *
 * derivative apps based on the code for noncommercial purposes only--so  *
 * long as you attribute the code to Deitel & Associates, Inc. and        *
 * reference www.deitel.com/books/iPhoneFP/. If you have any questions,   *
 * or specifically would like to use our code for commercial purposes,    *
 * contact deitel@deitel.com.                                             *
 *************************************************************************/

