// SlideshowDataViewController.m
// Manages the pictures, sounds and effects of a slideshow.
#import "SlideshowDataViewController.h"

@implementation SlideshowDataViewController

@synthesize pictures; // generate get method for pictures
@synthesize effect; // generate get method for effect
@synthesize music; // generate get method for music
@synthesize title; // generate get and set methods for title

// method is called when view finishes initializing
- (void)viewDidLoad
{
   [super viewDidLoad]; // initialize the superclass
   [self.navigationItem setTitle:@"Edit Slideshow"]; // set the bar title
   
   pictures = [[NSMutableArray alloc] init]; // initialize pictures
   
   // create the "Play" button
   // the "black translucent" style makes the button blue
   UIBarButtonItem *playButton = [[UIBarButtonItem alloc]
      initWithTitle:@"Play" style:UIBarStyleBlackTranslucent
      target:self action:@selector(startSlideshow)];
   
   // add the "Play" button to the top navigation bar on the right side
   [self.navigationItem setRightBarButtonItem:playButton];
   [playButton release]; // release the playButton UIBarButtonItem
   
   // create the toolbar at the bottom
   toolbar = [[UIToolbar alloc] init];
   [toolbar setBarStyle:UIBarStyleBlack]; // make toolbar black
   
   // create a "Add Picture" button for adding new photos
   UIBarButtonItem *pictureButton = [[UIBarButtonItem alloc]
      initWithTitle:@"Add Picture" style:UIBarStyleBlack target:self
      action:@selector(addPhoto)];
   
   // create the "Add Music" button
   UIBarButtonItem *musicButton = [[UIBarButtonItem alloc]
      initWithTitle:@"Add Music" style:UIBarStyleBlack target:self
      action:@selector(addMusic)];
   
   // create the "Add Effect" button
   UIBarButtonItem *effectButton = [[UIBarButtonItem alloc]
      initWithTitle:@"Set Effect" style:UIBarStyleBlack target:self
      action:@selector(addEffect)];
   
   // create a flexible space bar button item
   UIBarButtonItem *space = [[UIBarButtonItem alloc]
      initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace
      target:nil action:NULL];
   
   // add the buttons to toolbar in the given order
   [toolbar setItems:[NSArray arrayWithObjects:space, pictureButton,
      effectButton, musicButton, space, nil]];
   
   [pictureButton release]; // release the pictureButton UIBarButtonItem
   [musicButton release]; // release the musicButton UIBarButtonItem
   [effectButton release]; // release the effectButton UIBarButtonItem
   
   // add the toolbar to the superview
   [[self.navigationController view] addSubview:toolbar];
   
   [toolbar sizeToFit]; // expand the toolbar to include all the buttons
   CGRect frame = toolbar.frame; // get the frame of toolbar
   frame.origin.y = self.navigationController.view.bounds.size.height -
      toolbar.bounds.size.height; // move toolbar to the bottom edge
   [toolbar setFrame:frame]; // apply the new frame
   
   // put the table's cells in editing mode so they display reorder and
   // delete controls
   [(UITableView *)self.view setEditing:YES];
   
   // set the height of each row
   [(UITableView *)self.view setRowHeight:ROW_HEIGHT];
   
   firstLoad = YES; // this is the first load of this view
   returnFromImagePicker = NO; // we’re not returning from image picker
} // end method viewDidLoad

// called when view appears
- (void)viewDidAppear:(BOOL)animated
{
   [super viewDidAppear:animated]; // pass the message to the superclass
   
   // if this is the first time the view is appearing
   if (firstLoad)
   {
      // resize the table to fit the toolbar at the bottom
      firstLoad = NO; // the View has loaded once already
      CGRect frame = [self.view frame]; // fetch the frame of view
      
      // decrease the height of the table
      frame.size.height -= [toolbar bounds].size.height;
      [self.view setFrame:frame]; // apply the new frame
   } // end if
   else
   {
      // unhide the toolbar and navigation bar
      CGRect frame = toolbar.frame; // fetch the toolbar’s frame
      
      // set the frame to be just below the bottom of the screen
      frame.origin.y = self.navigationController.view.bounds.size.height;
      [toolbar setFrame:frame]; // apply the new frame
      [UIView beginAnimations:nil context:toolbar]; // begin animation
      frame.origin.y -= frame.size.height; // move the toolbar up
      [toolbar setFrame:frame]; // apply the new frame
      [UIView commitAnimations]; // end animation block
      
      // show the navigation bar at the top
      [self.navigationController setNavigationBarHidden:NO animated:YES];
   } // end else
} // end method viewDidAppear:

// called when the view is going to disappear
- (void)viewWillDisappear:(BOOL)animated
{
   [super viewWillDisappear:animated]; // pass message to the superclass
   
   // hide the toolbar
   [UIView beginAnimations:nil context:toolbar]; // begin animation block
   CGRect frame = [toolbar frame]; // fetch the frame of toolbar
   frame.origin.y += frame.size.height; // move the toolbar down
   [toolbar setFrame:frame]; // apply the new frame
   [UIView commitAnimations]; // end animation block
} // end method viewWillDisappear:

// add a new photo to the slideshow
- (void)addPhoto
{
   if (imagePicker == nil) // create the image picker
   {
      // initialize imagePicker
      imagePicker = [[UIImagePickerController alloc] init];
      imagePicker.allowsImageEditing = YES; // allow image editing
      
      // set the image source as the photo library
      imagePicker.sourceType =
         UIImagePickerControllerSourceTypePhotoLibrary;
      imagePicker.delegate = self; // set imagePicker's delegate
   } // end if
   
   returnFromImagePicker = YES; // we’re returning from image picker
   
   // show the image picker
   [self presentModalViewController:imagePicker animated:YES];
} // end method addPhoto

// add the chosen image to the slideshow
- (void)imagePickerController:(UIImagePickerController *)picker
   didFinishPickingImage:(UIImage *)img editingInfo:
   (NSDictionary *)editInfo
{
   [pictures addObject:img]; // add the picked image to the image list
   UITableView *table = (UITableView *)self.view;
   
   // insert a new row in the table for the new picture
   [table insertRowsAtIndexPaths:[NSArray arrayWithObject:[NSIndexPath
      indexPathForRow:pictures.count - 1 inSection:0]] withRowAnimation:
      UITableViewRowAnimationRight];
   
   // make the image picker go away
   [self dismissModalViewControllerAnimated:YES];
} // end method imagePickerController:didFinishPickingImage:

// called when the user touches the Add Music button
- (void)addMusic
{
   // create a new media picker configured for picking music
   musicPicker = [[MPMediaPickerController alloc] initWithMediaTypes:
      MPMediaTypeMusic];
   musicPicker.allowsPickingMultipleItems = YES;
   musicPicker.delegate = self;
   
   // show the music picker
   [self presentModalViewController:musicPicker animated:YES];
   [musicPicker release];
} // end method addMusic

// called when the user touches the done button in the media picker
- (void)mediaPicker: (MPMediaPickerController *)mediaPicker
  didPickMediaItems:(MPMediaItemCollection *)mediaItemCollection
{
   music = [mediaItemCollection retain]; // get the returned songs
   
   // make the media picker go away
   [self dismissModalViewControllerAnimated:YES];
} // end method mediaPicker:didPickMediaItems:

// called when the user touches the Add Effect button
- (void)addEffect
{
   if (effectSheet == nil) // first time the user touches the button
   {
      // create a new sheet with the given title and button titles
      effectSheet = [[UIActionSheet alloc] initWithTitle:@"Choose Effect"
         delegate:self cancelButtonTitle:nil destructiveButtonTitle:nil
         otherButtonTitles:@"Fade", @"Slide In", nil];
   } // end if
   
   [effectSheet showInView:self.view]; // show the sheet
} // end method addEffect

// called when the user touches one of the options in the effect sheet
- (void)actionSheet:(UIActionSheet *)actionSheet clickedButtonAtIndex:
   (NSInteger)buttonIndex
{
   effect = buttonIndex; // keep track of which effect is selected
} // end method actionSheet:clickedButtonAtIndex:

// called when the user touches the Start Slideshow button
- (void)startSlideshow
{
   // first time button was touched
   if (slideshowViewController == nil) 
      slideshowViewController = [[SlideshowViewController alloc] init];
   
   // set the pictures to appear in the slideshow
   slideshowViewController.pictures = pictures;
   slideshowViewController.effect = effect; // set the slideshow effect
   slideshowViewController.music = music; // set the slideshow music
   
   // hide the navigation bar so the screen is clear for the slideshow
   [self.navigationController setNavigationBarHidden:YES animated:YES];
   
   // show the view that plays the slideshow
   [self.navigationController pushViewController:slideshowViewController
      animated:YES];
} // end method startSlideshow

// called by RootViewController to get thumbnails for the table
- (UIImage *)firstImage
{
   // if no pictures are in the slideshow
   if (pictures.count == 0)
      return nil; // return nil
   
   return [pictures objectAtIndex:0]; // return the first picture
} // end method firstImage

// called by the table view to get the number of rows in a given section
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:
   (NSInteger)section
{
   return pictures.count;
} // end method tableView:numberOfRowsInSection:

// called by the table view to get the cells it needs to populate itself
- (UITableViewCell *)tableView:(UITableView *)tableView
   cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
   static NSString *CellIdentifier = @"Cell";
   
   // get a reused cell
   UITableViewCell *cell =
      [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
   
   // if no reusable cells were available
   if (cell == nil)
      
      // create a new cell
      cell = [[[UITableViewCell alloc] initWithStyle:
         UITableViewCellStyleDefault reuseIdentifier:CellIdentifier]
         autorelease];
   
   // remove any views that exist in a reused cell
   for (UIView *view in cell.contentView.subviews)
      [view removeFromSuperview];
   
   // get the image for the given row
   UIImage *image = [pictures objectAtIndex:indexPath.row];
   
   // create an image view for the image
   UIImageView *view = [[UIImageView alloc] initWithImage:image];
   
   // resize the image without distorting it
   float newWidth = image.size.width * ROW_HEIGHT / image.size.height;
   CGRect frame;
   
   // create the image shifted to the left of the center by 50 pts
   frame.origin.x = cell.center.x - newWidth / 2 - 50;
   frame.origin.y = 0; // the image will fill the height of the cell
   frame.size.width = newWidth; // the width so there is no distortion
   frame.size.height = ROW_HEIGHT; // the image will fill the whole height
   view.frame = frame; // assign the new frame
   [cell.contentView addSubview:view]; // add the image to the cell
   
   return cell; // return the configured cell
} // end method tableView:cellForRowAtIndexPath:

// called by the table when the user touches the delete button
- (void)tableView:(UITableView *)tableView commitEditingStyle:
   (UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:
   (NSIndexPath *)indexPath
{
   // if the user touched a delete button
   if (editingStyle == UITableViewCellEditingStyleDelete)
   {
      // remove the object at the deleted row
      [pictures removeObjectAtIndex:indexPath.row];
      
      // remove the row from the table
      [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:
         indexPath] withRowAnimation:UITableViewRowAnimationLeft];
   } // end if
} // end method tableView:commitEditingStyle:forRowAtIndexPath:

- (void)tableView:(UITableView *)tableView moveRowAtIndexPath:
   (NSIndexPath *)fromIndexPath toIndexPath:(NSIndexPath *)toIndexPath
{
   // get the image at the moved row
   UIImage *image = [pictures objectAtIndex:fromIndexPath.row];
   [pictures removeObject:image]; // remove the image from the list
   
   // insert the image into the list at the specified index
   [pictures insertObject:image atIndex:toIndexPath.row];
} // end method tableView:moveRowAtIndexPath:toIndexPath:

// called by the table view to check if a given row can be moved
- (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:
   (NSIndexPath *)indexPath
{
   return YES; // all the rows in this table can be moved
} // end method tableView:canMoveRowAtIndexPath:

// release this object's memory
- (void)dealloc
{
   [imagePicker release]; // release imagePicker UIImagePickerController
   [slideshowViewController release]; // release slideshowViewController
   [pictures release]; // release the pictures NSMutableArray
   [toolbar release]; // release the toolbar UIToolbar
   [music release]; // release the music MPMediaItemCollection
   [super dealloc]; // call the superclass's dealloc method
} // end method dealloc
@end // end SlideshowDataViewController class


/**************************************************************************
 * (C) Copyright 2010 by Deitel & Associates, Inc. All Rights Reserved.   *
 *                                                                        *
 * DISCLAIMER: The authors and publisher of this book have used their     *
 * best efforts in preparing the book. These efforts include the          *
 * development, research, and testing of the theories and programs        *
 * to determine their effectiveness. The authors and publisher make       *
 * no warranty of any kind, expressed or implied, with regard to these    *
 * programs or to the documentation contained in these books. The authors *
 * and publisher shall not be liable in any event for incidental or       *
 * consequential damages in connection with, or arising out of, the       *
 * furnishing, performance, or use of these programs.                     *
 *                                                                        *
 * As a user of the book, Deitel & Associates, Inc. grants you the        *
 * nonexclusive right to copy, distribute, display the code, and create   *
 * derivative apps based on the code for noncommercial purposes only--so  *
 * long as you attribute the code to Deitel & Associates, Inc. and        *
 * reference www.deitel.com/books/iPhoneFP/. If you have any questions,   *
 * or specifically would like to use our code for commercial purposes,    *
 * contact deitel@deitel.com.                                             *
 *************************************************************************/

