// SlideshowViewController.m
// Controller for a view that shows a slideshow
#import "SlideshowViewController.h"

@implementation SlideshowViewController

@synthesize pictures; // generate getter and setter for pictures
@synthesize effect; // generate getter and setter for effect
@synthesize music; // generate getter and setter for music

// setup the view
- (void)loadView
{
   self.view = [[UIView alloc] init]; // initialize the main view
   
   // size the main view to fill the entire screen
   self.view.frame = [UIScreen mainScreen].bounds;
} // end method loadView

// Returns a UIImageView that contains the next image to display
- (UIImageView *)nextImageView
{
   // get the image at the next index
   UIImage *image = [pictures objectAtIndex:pictureIndex];
   ++pictureIndex; // increment the index
   
   // create an image view for the image
   UIImageView *imageView = [[UIImageView alloc] initWithImage:image];
   
   CGRect screenBounds = self.view.bounds; // get the screen bounds
   
   // resize the image to fill the screen without distorting
   [imageView expandToFill:screenBounds];

   // position the image to appear in the center of the view
   imageView.center = self.view.center;
   
   // Makes the image move proportionally in any direction if the
   // bounds of the superview change when the iPhone is rotated.
   imageView.autoresizingMask = (UIViewAutoresizingFlexibleLeftMargin |
      UIViewAutoresizingFlexibleRightMargin |                          
      UIViewAutoresizingFlexibleTopMargin |                            
      UIViewAutoresizingFlexibleBottomMargin);
   
   return imageView;
} // end method nextImageView

// called from timerFired: if there are no more images to be displayed
- (void)exitShow
{
   [musicPlayer stop]; // stop the music
   
   // display the status bar
   [[UIApplication sharedApplication] setStatusBarHidden:NO];
   
   // remove the slideshow view to return to the previous View
   [self.navigationController popViewControllerAnimated:YES];
} // end method exitShow

// called every five seconds when the timer fires
- (void)timerFired:(NSTimer *)timer
{
   // check if there’s another image to display
   if (pictureIndex >= pictures.count)
   {
      [self exitShow]; // if there’s no image, exit the slideshow
   } // end if
   else
   {
      // get the next image to display
      UIImageView *nextImageView = [self nextImageView];
      [self.view addSubview:nextImageView]; // add the image to the view
      CGRect frame;
      
      // set the next image to its beginning state for the effect
      switch (effect)
      {
         case TransitionEffectFade: // the user chose the fade effect
            nextImageView.alpha = 0.0; // make the next image transparent
            break;
         case TransitionEffectSlide: // the user chose the slide effect
            
            // position the next image to the right of the screen
            frame = nextImageView.frame; // get the next image’s frame
            frame.origin.x += frame.size.width; // move frame off screen
            nextImageView.frame = frame; // apply the repositioned frame
            break;
      } // end switch
      
      // begin animation block
      [UIView beginAnimations:nil context:nextImageView];
      [UIView setAnimationDuration:2.0]; // set the animation length
      [UIView setAnimationDelegate:self]; // set the animation delegate
      
      // call the given method when the animation ends
      [UIView setAnimationDidStopSelector:
         @selector(transitionFinished:finished:context:)];
      
      // make the next image appear with the chosen effect
      switch (effect)
      {
         case TransitionEffectFade: // the user chose the fade effect
            [nextImageView setAlpha:1.0]; // fade in the next image
            [currentImageView setAlpha:0.0]; // fade out the old image
            break;
         case TransitionEffectSlide: // the user chose the slide effect
            frame.origin.x -= frame.size.width; // slide new image left
            nextImageView.frame = frame; // apply the repositioned frame
            CGRect currentImageFrame = currentImageView.frame;
            
            // slide the old image to the left
            currentImageFrame.origin.x -= currentImageFrame.size.width;
            currentImageView.frame = currentImageFrame; // apply frame
            break;
      } // end switch
      
      [UIView commitAnimations]; // end animation block
   } // end else
} // end method timerFired:

// called when the image transition animation finishes
- (void)transitionFinished:(NSString *)animationId finished:(BOOL)finished
   context:(void *)context
{
   [currentImageView removeFromSuperview]; // remove the old image
   [currentImageView release]; // release the memory for the old image
   currentImageView = context; // assign the new image
} // end method transitionFinished:finished:context:

// called when the View appears
- (void)viewWillAppear:(BOOL)animated
{
   [super viewWillAppear:animated]; // pass the message to the superclass
   pictureIndex = 0; // reset the index
   
   currentImageView = [self nextImageView]; // load the first image
   [self.view addSubview:currentImageView]; // add the image to the view
   
   // hide the status bar so the slideshow can appear fullscreen
   [[UIApplication sharedApplication] setStatusBarHidden:YES];
   
   // initialize the timer to fire every 5 seconds
   timer = [NSTimer scheduledTimerWithTimeInterval:5.0 target:self
      selector:@selector(timerFired:) userInfo:nil repeats:YES];
   
   // if the user has selected music to play
   if (music != nil)
   {
      // get the application music player
      musicPlayer = [MPMusicPlayerController applicationMusicPlayer];     
      musicPlayer.shuffleMode = MPMusicShuffleModeOff; // turn off shuffle
      musicPlayer.repeatMode = MPMusicRepeatModeNone; // turn off repeat  
      
      // add the music the user selected to the queue
      [musicPlayer setQueueWithItemCollection:music];
      [musicPlayer play]; // play the music          
   } // end if
} // end method viewWillAppear:

// called when the View disappears
- (void)viewDidDisappear:(BOOL)animated
{
   [super viewDidDisappear:animated];
   
   // Stop the timer
   [timer invalidate];
   timer = nil; // set timer to nil
   [currentImageView removeFromSuperview]; // remove the current image
} // end method viewDidDisappear:

// determines whether the view rotates when the iPhone orientation changes
- (BOOL)shouldAutorotateToInterfaceOrientation:
   (UIInterfaceOrientation)interfaceOrientation
{
   return YES; // allow rotation to all interface orientations
} // end method shouldAutorotateToInterfaceOrientation

// called when the iPhone orientation changes
- (void)willRotateToInterfaceOrientation:               
   (UIInterfaceOrientation)interfaceOrientation duration:
   (NSTimeInterval)duration                              
{
   // create a CGRect with the view's height and width flipped
   CGRect bounds = CGRectMake(0, 0, self.view.bounds.size.height,
      self.view.bounds.size.width);
   
   // make the current image resize to fill the flipped bounds
   [currentImageView expandToFill:bounds];
} // end method willRotateToInterfaceOrientation:

// free SlideshowViewController's memory
- (void)dealloc
{
   [currentImageView release]; // release the currentImageView UIImageView
   [super dealloc]; // call the superclass's dealloc method
} // end method dealloc
@end // end implementation SlideshowviewController

@implementation UIImageView (Scaling) // extensions to UIImageView

// scale the view to fill the given bounds without distorting
- (void)expandToFill:(CGRect)bounds
{
   UIImage *image = self.image; // get the image of this view
   CGRect frame = self.frame; // get the frame of this view
   
   // check if the image is bound by its height
   if (image.size.height / image.size.width >
       bounds.size.height / bounds.size.width)
   {
      // expand the new height to fill the entire view
      frame.size.height = bounds.size.height;
      
      // calculate the new width so the image isn't distorted
      frame.size.width = image.size.width * bounds.size.height /
         image.size.height;
      
      // add to the x and y coordinates so the view remains centered
      frame.origin.y += (self.frame.size.height - frame.size.height) / 2;
      frame.origin.x += (self.frame.size.width - frame.size.width) / 2;
   } // end if
   else // the image is bound by its width
   {
      // expand the new width to fill the entire view
      frame.size.width = bounds.size.width;
      
      // calculate the new height so the image isn't distorted
      frame.size.height = image.size.height * bounds.size.width /
         image.size.width;
      
      // add to the x and y coordinates so the view remains centered
      frame.origin.y += (self.frame.size.height - frame.size.height) / 2;
      frame.origin.x += (self.frame.size.width - frame.size.width) / 2;
   } // end else
   
   self.frame = frame; // assign the new frame
} // end method expandToFill:
@end // end implementation UIImageView (Scaling)


/**************************************************************************
 * (C) Copyright 2010 by Deitel & Associates, Inc. All Rights Reserved.   *
 *                                                                        *
 * DISCLAIMER: The authors and publisher of this book have used their     *
 * best efforts in preparing the book. These efforts include the          *
 * development, research, and testing of the theories and programs        *
 * to determine their effectiveness. The authors and publisher make       *
 * no warranty of any kind, expressed or implied, with regard to these    *
 * programs or to the documentation contained in these books. The authors *
 * and publisher shall not be liable in any event for incidental or       *
 * consequential damages in connection with, or arising out of, the       *
 * furnishing, performance, or use of these programs.                     *
 *                                                                        *
 * As a user of the book, Deitel & Associates, Inc. grants you the        *
 * nonexclusive right to copy, distribute, display the code, and create   *
 * derivative apps based on the code for noncommercial purposes only--so  *
 * long as you attribute the code to Deitel & Associates, Inc. and        *
 * reference www.deitel.com/books/iPhoneFP/. If you have any questions,   *
 * or specifically would like to use our code for commercial purposes,    *
 * contact deitel@deitel.com.                                             *
 *************************************************************************/

