// SpotOnViewController.m
// Controller for the Spot On Game app
#import "SpotOnViewController.h"
static const int BALL_RADIUS = 40;

@implementation SpotOnViewController

// generate get and set methods for our properties
@synthesize scoreLabel;
@synthesize levelLabel;
@synthesize highScoreLabel;

// called when the view finishes loading
- (void)viewDidLoad
{
   [super viewDidLoad]; // pass the message to the superclass
   srandom(time(0)); // seed random number generation
   spots = [[NSMutableArray alloc] init]; // initialize spots
   lives = [[NSMutableArray alloc] init]; // initialize lives
   
   // initialize the two spot images
   touchedImage = [UIImage imageNamed:@"touched.png"];
   untouchedImage = [UIImage imageNamed:@"untouched.png"];
   
   // get the path of the sound file
   NSString *soundPath = [[NSBundle mainBundle] pathForResource:@"hit"
      ofType:@"wav"];
   
   // create a URL with the given path
   NSURL *fileURL = [[NSURL alloc] initFileURLWithPath:soundPath];
   
   // initialize the AVAudioPlayer with the sound file
   hitPlayer = [[AVAudioPlayer alloc] initWithContentsOfURL:fileURL
      error:nil];
   hitPlayer.volume = 0.3; // set hitPlayer's volume
   
   // get the path of the sound file
   soundPath = [[NSBundle mainBundle] pathForResource:@"miss"
      ofType:@"wav"];
   
   // create a URL with the given path
   [fileURL release];
   fileURL = [[NSURL alloc] initFileURLWithPath:soundPath];
   
   // initialize the AVAudioPlayer with the sound file
   missPlayer = [[AVAudioPlayer alloc] initWithContentsOfURL:fileURL
      error:nil];
   missPlayer.volume = 0.7; // set missPlayer's volume
   
   // get the path of the sound file
   soundPath =
      [[NSBundle mainBundle] pathForResource:@"disappear" ofType:@"wav"];
   
   // create a URL with the given path
   [fileURL release];
   fileURL = [[NSURL alloc] initFileURLWithPath:soundPath];
   
   // initialize the AVAudioPlayer with the sound file
   disappearPlayer =
      [[AVAudioPlayer alloc] initWithContentsOfURL:fileURL error:nil];
   disappearPlayer.volume = 0.3; // set disappearPlayer's volume
   [fileURL release]; // release the fileURL NSURL
   
   [self resetGame]; // begin a new game
} // end method viewDidLoad

// removes old objects and begins a new game
- (void)resetGame
{   
   [spots removeAllObjects]; // empty the array of spots
   drawTime = 3.0; // reset the draw time
   spotsTouched = 0; // reset the number of spots touched
   score = 0; // reset the score
   level = 1; // reset the level
   [scoreLabel setText:@"Score: 0"]; // reset the score label
   [levelLabel setText:@"Level: 1"]; // reset the level label
   
   // get the high score from the preferences file
   NSNumber *highScore =
      [[NSUserDefaults standardUserDefaults] valueForKey:@"highScore"];
   
   // if there isn't a current high score
   if (highScore == nil)
      highScore = [NSNumber numberWithInt:0]; // set the high score to 0
   
   // update the high score label with the high score
   [highScoreLabel setText:
      [NSString stringWithFormat:@"High score: %@", highScore]];
   gameOver = NO; // reset the gameOver boolean
   
   // add three lives
   for (int i = 0; i < 3; i++)
   {
      UIImageView *life = [[UIImageView alloc] 
         initWithImage:touchedImage];
      
      // position the views next to each other at the bottom left
      CGRect frame = CGRectMake(10 + 40 * i, 420, 30, 30);
      life.frame = frame; // assign the new frame
      [lives addObject:life]; // add the view to the list of lives
      [self.view addSubview:life]; // add the image to the view
      [life release];
   } // end for
   
   // add three new spots, each spaced by a one-second delay
   [self addNewSpot];
   [self performSelector:@selector(addNewSpot) withObject:nil
      afterDelay:1.0]; // call addNewSpot after 1 second
   [self performSelector:@selector(addNewSpot) withObject:nil
      afterDelay:2.0]; // call addNewSpot after 2 seconds
} // end method resetGame

// adds a new spot at a random location
- (void)addNewSpot
{
   // get the view width and height
   float viewWidth = self.view.bounds.size.width;
   float viewHeight = self.view.bounds.size.height;
   
   // pick random coordinates inside the view
   float x = random() % (int)(viewWidth - 2 * BALL_RADIUS);
   float y = random() % (int)(viewHeight - 2 * BALL_RADIUS);
   
   // create a new spot
   UIImageView *spot = [[UIImageView alloc] initWithImage:untouchedImage]; 
   [spots addObject:spot]; // add the spot to the spots NSMutableArray
   [self.view addSubview:spot]; // add the spot to the main view
   
   // set the frame of variable spot with the random coordinates
   [spot setFrame:CGRectMake(x, y, BALL_RADIUS * 2, BALL_RADIUS * 2)];
   
   // delay beginning animation to give the spot time to redraw
   [self performSelector:@selector(beginSpotAnimation:) withObject:spot
      afterDelay:0.01];
   
   [spot release]; // release the spot UIImageView
} // end method addNewSpot

// start the animation of the given spot
- (void)beginSpotAnimation:(UIImageView *)spot
{
   // get the width and height of view
   float viewWidth = [self.view bounds].size.width;
   float viewHeight = [self.view bounds].size.height;
   
   // pick random coordinates inside the view
   float x = random() % (int)(viewWidth - 2 * BALL_RADIUS);
   float y = random() % (int)(viewHeight - 2 * BALL_RADIUS);
   
   [UIView beginAnimations:nil context:spot]; // begin the animation block
   [UIView setAnimationDelegate:self]; // set the delegate as this object
   
   // call the given method of the delegate when the animation ends
   [UIView setAnimationDidStopSelector:
      @selector(finishedAnimation:finished:context:)];
   [UIView setAnimationDuration:drawTime]; // set the animation length
   
   // make the animation start slow and speed up
   [UIView setAnimationCurve:UIViewAnimationCurveEaseIn];
   
   // set the ending location of the spot
   [spot setFrame:CGRectMake(x + BALL_RADIUS, y + BALL_RADIUS, 0, 0)];
   [UIView commitAnimations]; // end the animation block
} // end method beginSpotAnimation:

// method is called when the player touches the screen
- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event
{
   BOOL hitSpot = NO; // initialize hitSpot
   
   for (UITouch *touch in touches) // loop through all the new touches
   {
      // get the location of the touch in the main view
      CGPoint point = [touch locationInView:self.view];
      
      // loop through all the spots to check if the player hit any;
      // iterate backwards so foreground spots get checked first
      for (int i = spots.count - 1; i >= 0 && !hitSpot; i--)
      {
         UIImageView *spot = [spots objectAtIndex:i];
         
         // We need to get the current location of spot, but the frame
         // of spot is already set to its ending location. To get the
         // displayed frame, we need to access the Core Animation layer.
         CGRect frame = [[spot.layer presentationLayer] frame];
         
         // compute the point at the center of the spot
         CGPoint origin = CGPointMake(frame.origin.x + frame.size.width /
            2, frame.origin.y + frame.size.height / 2);

         // compute the distance between the spot's center and the touch
         float distance = 
            pow(origin.x - point.x, 2) + pow(origin.y - point.y, 2);
         distance = sqrt(distance); // square root to complete formula
         
         // check if the touch is within the spot
         if (distance <= frame.size.width / 2)
         {
            spot.image = touchedImage; // change to touched spot image
            [self touchedSpot:spot]; // call the touchedSpot: method
            hitSpot = YES; // a spot has been touched
         } // end if
      } // end for
   } // end for
   
   if (!hitSpot) // if the player missed
   {
      [missPlayer play]; // play the miss sound
      score -= 20 * level; // remove some points
      
      if(score < 0)
         score = 0;
      
      // update the score label
      [scoreLabel setText:
         [NSString stringWithFormat:@"Score: %i", score]];
   } // end if
} // end method touchesBegan:withEvent:

- (void)touchedSpot:(UIImageView *)spot
{	
   ++spotsTouched; // increment the number of spots touched
   score += 10 * level; // increment the score
   hitPlayer.currentTime = 0; // reset the playback to the beginning
   [hitPlayer play]; // play the sound for when the player hits a spot
   
   // update the score label
   [scoreLabel setText:[NSString stringWithFormat:@"Score: %i", score]];
   
   // increment level if the player touched 10 spots in the current level
   if (spotsTouched % 10 == 0)
   {
      ++level; // increment the level
      drawTime *= 0.95; // speed up the game
      
      // update the level label
      [levelLabel setText:
         [NSString stringWithFormat:@"Level: %i", level]];
      
      // add a new life if it fits on the screen
      if (lives.count < 7)
      {
         // load the life image
         UIImageView *life = [[UIImageView alloc] 
            initWithImage:touchedImage];
         
         // position the View to the right of the previous life View
         CGRect frame = CGRectMake(10 + 40 * lives.count, 420, 30, 30);
         life.frame = frame; // set life's frame
         [self.view addSubview:life]; // add life to View
         [lives addObject:life]; // add life to lives array
         [life release]; // release the life UIImageView
      } // end if
   } // end if
   
   // stop the current animation and start a new one at the same place
   CGRect frame = [[spot.layer presentationLayer] frame]; // get the frame
   [spot.layer removeAllAnimations]; // stop the animation
   spot.frame = frame; // move the spot to where the old animation ended
   [spot setNeedsDisplay]; // redraw the spot
   
   // give the spot time to redraw by delaying the end animation
   [self performSelector:@selector(beginSpotEndAnimation:) withObject:spot
      afterDelay:0.01];
} // end method touchedSpot:

// starts the fade-away animation for a spot that's been touched
- (void)beginSpotEndAnimation:(UIImageView *)spot
{
   [UIView beginAnimations:@"end" context:spot]; // begin animation block
   [UIView setAnimationDuration:0.8]; // set the time for the animation
   [UIView setAnimationDelegate:self]; // set the animation delegate
   
   // set the method to call when the animation ends
   [UIView setAnimationDidStopSelector:
      @selector(finishedAnimation:finished:context:)];
   
   // make the spot stay in the same place and disappear
   CGRect frame = spot.frame; // get the current frame
   frame.origin.x += frame.size.width / 2; // set x to the center
   frame.origin.y += frame.size.height / 2; // set y to the center
   frame.size.width = 0; // set the width to 0
   frame.size.height = 0; // set the height to 0
   [spot setFrame:frame]; // assign the new frame
   [spot setAlpha:0.0]; // set the spot to be fully transparent
   [UIView commitAnimations]; // end the animation block
} // end method beginSpotEndAnimation:

// method is automatically called when an animation ends
- (void)finishedAnimation:(NSString *)animationId finished:(BOOL)finished
   context:(void *)context
{	
   // if the game has already been lost, exit
   if (gameOver)
      return;
   
   UIImageView *spot = (UIImageView *)context; // get the finished spot
   
   // if it was an ending animation that finished, add a new spot
   if (animationId == @"end")
   {
      // remove spot from the spots NSMutableArray
      [spots removeObject:spot];  
      [spot removeFromSuperview]; // remove the old spot
      [self addNewSpot]; // add a new spot
   } // end if
   else if ([spot.image isEqual:untouchedImage]) // a spot was missed
   {
      [disappearPlayer play]; // play disappearing spot sound effect
      
      // the game has been lost
      if (lives.count == 0)
      {
         for (UIImageView *spot in spots) // delete all old spots
         {
            [spot removeFromSuperview]; // remove the spot from the View
            [spot.layer removeAllAnimations]; // stop existing animations
         } // end for
         
         gameOver = YES; // the game is over
         
         // display the game-over alert
         NSString *message =
            [NSString stringWithFormat:@"Score: %i", score];
         
         // get the standard user defaults
         NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
         
         // get the current high score
         int highScore = [[defaults valueForKey:@"highScore"] intValue];
         
         // if the score of the last game is greater than the high score
         if (score > highScore)
            
            // update the file with the new high score
            [defaults setValue:[NSNumber numberWithInt:score]
               forKey:@"highScore"];
         
         UIAlertView *alert = [[UIAlertView alloc]
            initWithTitle:@"Game Over" message:message delegate:self
            cancelButtonTitle:@"Reset Game" otherButtonTitles:nil];
         [alert show]; // show the alert
         [alert release]; // release the alert UIAlertView
      } // end else
      else // remove one life
      {
         UIImageView *life = [lives lastObject]; // get the last life
         [lives removeLastObject]; // remove the life from the array
         [life removeFromSuperview]; // remove the life from the screen
         
         // remove the old spot and create a new one
         [spot removeFromSuperview]; // remove old spot
         [self addNewSpot]; // add a new spot
      } // end else
   } // end else
} // end method finishedAnimation:finished:context:

// called when the player touches the "Reset Game" button
- (void)alertView:(UIAlertView *)alertView
   clickedButtonAtIndex:(NSInteger)buttonIndex
{
   [self resetGame]; // create a new game
} // end method alertView:clickedButtonAtIndex:

// called to determine what orientations our View allows
- (BOOL)shouldAutorotateToInterfaceOrientation:
   (UIInterfaceOrientation)interfaceOrientation
{
   // allow only the portrait orientation
   return (interfaceOrientation == UIInterfaceOrientationPortrait);
} // end method shouldAutorotateToInterfaceOrientation:

// free SpotOnViewController's memory
- (void)dealloc
{
   [spots release]; // release the spots NSMutableArray
   [lives release]; // release the lives NSMutableArray
   [hitPlayer release]; // release the hitPlayer AVAudioPlayer
   [missPlayer release]; // release the missPlayer AVAudioPlayer
   [disappearPlayer release]; // release the disappearPlayer AVAudioPlayer
   [scoreLabel release]; // release the scoreLabel UILabel
   [levelLabel release]; // release the levelLabe UILabel
   [highScoreLabel release]; // release the highScoreLabel UILabel
   [super dealloc]; // invokes the superclass's dealloc method
} // end method dealloc
@end

/**************************************************************************
 * (C) Copyright 2010 by Deitel & Associates, Inc. All Rights Reserved.   *
 *                                                                        *
 * DISCLAIMER: The authors and publisher of this book have used their     *
 * best efforts in preparing the book. These efforts include the          *
 * development, research, and testing of the theories and programs        *
 * to determine their effectiveness. The authors and publisher make       *
 * no warranty of any kind, expressed or implied, with regard to these    *
 * programs or to the documentation contained in these books. The authors *
 * and publisher shall not be liable in any event for incidental or       *
 * consequential damages in connection with, or arising out of, the       *
 * furnishing, performance, or use of these programs.                     *
 *                                                                        *
 * As a user of the book, Deitel & Associates, Inc. grants you the        *
 * nonexclusive right to copy, distribute, display the code, and create   *
 * derivative apps based on the code for noncommercial purposes only--so  *
 * long as you attribute the code to Deitel & Associates, Inc. and        *
 * reference www.deitel.com/books/iPhoneFP/. If you have any questions,   *
 * or specifically would like to use our code for commercial purposes,    *
 * contact deitel@deitel.com.                                             *
 *************************************************************************/

