// Fig. 14.22: PlaybackViewController.m
// Implementation for PlaybackViewController.
#import <AVFoundation/AVFoundation.h>
#import "PlaybackViewController.h"

@implementation PlaybackViewController

@synthesize delegate; // generate get and set methods for the delegate
@synthesize table; // generate get and set methods for table
@synthesize toolbar; // generate get and set methods for toolbar
@synthesize progressSlider; // generate get and set methods for the slider
@synthesize volumeSlider; // generate get and set methods for the slider
@synthesize timeLabel; // generate get and set methods for timeLabel
@synthesize playButton; // generate get and set methods for playButton

// setup the view
- (void)viewDidLoad
{
   files = [[NSMutableArray alloc] init]; // initialize files
   
   // find the directory that the recordings are saved in
   NSArray *paths = NSSearchPathForDirectoriesInDomains(
      NSDocumentDirectory, NSUserDomainMask, YES);
   NSString *dir = [paths objectAtIndex:0];
   
   // get the default file manager
   NSFileManager *filemanager = [NSFileManager defaultManager];
   
   // get a list of all the files in the directory
   NSArray *filelist = [filemanager directoryContentsAtPath:dir];
   
   // iterate through each file in the directory
   for (NSString *file in filelist)
   {
      // if the file's extension is "caf"
      if ([[file pathExtension] isEqualToString:@"caf"])
         // add the path to files
         [files addObject:[dir stringByAppendingPathComponent:file]];
   } // end for
   
   [table reloadData]; // refresh the table
} // end method viewDidLoad

// called when the user moves the playback slider
- (IBAction)sliderMoved:sender
{
   // if the player is currently playing a recording
   if (player != nil)
      // update the player's playback time
      player.currentTime = progressSlider.value;
} // end method sliderMoved:

// called when the user touches the play/pause button
- (IBAction)togglePlay:sender
{
   // if the player is playing a recording
   if (player.playing)
      [self stopSound]; // stop playback
   else if (player != nil) // if the player has been created
      [self playSound]; // play the player's sound
} // end method togglePlay:

// called when the user moved the volume slider
- (IBAction)updateVolume:sender
{
   player.volume = volumeSlider.value; // update player's volume
} // end method updateVolume:

// update progressSlider and timeLabel when the timer fires
- (void)timerFired:(NSTimer *)t
{
   // if the player is playing
   if (player.playing)
   {
      double time = player.currentTime; // get the current playback time
      
      // update progressSlider with the time
      [progressSlider setValue:time animated:NO];
      
      // update timeLabel with the time in minutes:seconds
      timeLabel.title = [NSString stringWithFormat:@"%i:%.02i",
                         (int)time / 60, (int)time % 60];
   } // end if
   else // if the player isn’t playing
   {
      [self stopSound]; // stop the playback
      [progressSlider setValue:0 animated:YES]; // move slider to start
   } // end else
} // end method timerFired:

// called when the user touches the "Record" Button
- (IBAction)record:sender
{
   // inform the delegate that the user is done playing recordings
   [delegate playbackViewControllerDidFinish:self];
} // end method record:

// plays the current recording
- (void)playSound
{
   // set the audio session's category to playback
   [[AVAudioSession sharedInstance] setCategory:
    AVAudioSessionCategoryPlayback error:nil];
   [player play]; // play the audio player
   
   // initialize the timer
   timer = [NSTimer scheduledTimerWithTimeInterval:0.1 target:self
      selector:@selector(timerFired:) userInfo:nil repeats:YES];
   
   // create a pause button
   UIBarButtonItem *pauseButton = [[UIBarButtonItem alloc]
      initWithBarButtonSystemItem:UIBarButtonSystemItemPause target:self
      action:@selector(togglePlay:)];
   
   // get the items in the toolbar
   NSMutableArray *items = [toolbar.items mutableCopy];
   
   // replace the play button with the pause button
   [items removeObjectAtIndex:0]; // remove the play button
   [items insertObject:pauseButton atIndex:0]; // add the pause button
   [pauseButton release]; // release the pauseButton UIBarButtonItem
   [toolbar setItems:items animated:NO]; // update toolbar's items
} // end method playSound

// stops the playback of current recording
- (void)stopSound
{
   [timer invalidate]; // stop and release the timer
   timer = nil; // assign the timer to nil
   [player pause]; // pause the audio player
   
   // set the audio session's category to ambient
   [[AVAudioSession sharedInstance] setCategory:   
    AVAudioSessionCategorySoloAmbient error:nil];
   
   // get the items in the toolbar
   NSMutableArray *items = [toolbar.items mutableCopy];
   
   // replace the pause button with the play button
   [items removeObjectAtIndex:0]; // remove the pause button
   [items insertObject:playButton atIndex:0]; // add the play button
   [toolbar setItems:items animated:NO]; // update the toolbar's items
} // end method stopSound

// called by the table view to determine its number of sections
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
   return 1; // this table has only one section
} // end method numberOfSectionsInTableView:

// called by the table view to determine the number of rows in a section
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:
(NSInteger)section
{
   return files.count; // return the number of files
} // end method tableView:numberOfRowsInSection:

// called by the table view to get a cell for the given index path
- (UITableViewCell *)tableView:(UITableView *)tableView
   cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
   static NSString *ID = @"Cell"; // create a cell identifier
   
   // get a reused cell using the identifier
   UITableViewCell *cell =
   [tableView dequeueReusableCellWithIdentifier:ID];
   
   // if there were no cells available for reuse
   if (cell == nil)
      // create a new cell
      cell = [[[UITableViewCell alloc] initWithStyle:
               UITableViewCellStyleDefault reuseIdentifier:ID] autorelease];
   
   // get the file name for this cell
   NSString *text =
   [[files objectAtIndex:indexPath.row] lastPathComponent];
   
   // delete the .caf from the path
   cell.textLabel.text = [text stringByDeletingPathExtension];
   
   // load the email icon
   UIImage *mailImage = [UIImage imageNamed:@"envelope.png"];
   
   // create a new button
   UIButton *mailButton =
      [[UIButton alloc] initWithFrame:CGRectMake(0, 0, 32, 32)];
   
   // use the email icon for the button
   [mailButton setImage:mailImage forState:UIControlStateNormal];
   mailButton.tag = indexPath.row; // tag the button with the current row
   
   // make the button call the mailButtonTouched: method when touched
   [mailButton addTarget:self action:@selector(mailButtonTouched:)
        forControlEvents:UIControlEventTouchUpInside];
   
   // make the button the accessory view of the cell
   cell.accessoryView = mailButton;
   return cell; // return the configured cell
} // end method tableView:cellForRowAtIndexPath:

// called when the user edits the table view
- (void)tableView:(UITableView *)tableView commitEditingStyle:
(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:
(NSIndexPath *)indexPath
{
   // if the user deleted the element
   if (editingStyle == UITableViewCellEditingStyleDelete)
   {
      // get the default file manager
      NSFileManager *fileManager = [NSFileManager defaultManager];
      
      // get the path for the deleted recording
      NSString *path = [files objectAtIndex:indexPath.row];
      
      // if the recording being deleted is also being played
      if ([[player.url path] isEqualToString:path])
      {
         [self stopSound]; // stop the playback
         [player release]; // release the player AVAudioPlayer
         player = nil; // set player to nil
      } // end if
      
      [fileManager removeItemAtPath:path error:nil]; // delete recording
      
      // remove the entry from files
      [files removeObjectAtIndex:indexPath.row];
      
      // remove the row from the table
      [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:
         indexPath] withRowAnimation:UITableViewRowAnimationRight];
   }
} // end method tableView:commitEditingStyle:forRowAtIndexPath:

// called when the user touches a row of the table view
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:
(NSIndexPath *)indexPath
{
   // get the file name for the touches row
   NSString *file = [files objectAtIndex:indexPath.row];
   
   // create a URL with the file's path
   NSURL *url = [NSURL URLWithString:file];
   [player release]; // release the player AVAudioPlayer
   
   // create a new AVAudioPlayer with the URL
   player = [[AVAudioPlayer alloc] initWithContentsOfURL:url error:nil];
   player.volume = volumeSlider.value; // set player's volume           
   
   // set the maximum value of the slider to reflect the recording length
   progressSlider.maximumValue = player.duration;
   [self playSound]; // play the selected recording
   playButton.enabled = YES; // enable the play/pause button
} // end method tableView:didSelectRowAtIndexPath:

// called when the user touches the mail button on a cell
- (void)mailButtonTouched:sender
{
   // get the file for the touched row
   NSString *file = [files objectAtIndex:[sender tag]];
   
   // create an NSData object with the selected recording
   NSData *data = [NSData dataWithContentsOfFile:file];
   
   // create a MFMailComposeViewController for sending an e-mail
   MFMailComposeViewController *controller =     
   [[MFMailComposeViewController alloc] init];
   
   // add the recording as an attachment
   [controller addAttachmentData:data mimeType:@"audio/mp4" fileName:
    [file lastPathComponent]];                                     
   
   // set controller's delegate to this object
   controller.mailComposeDelegate = self;
   
   // show the MFMailComposeViewController
   [self presentModalViewController:controller animated:YES];
} // end method tableView:accessoryButtonTappedForRowWithIndexPath:

// called when the user finishes sending an e-mail
- (void)mailComposeController:(MFMailComposeViewController*)controller
   didFinishWithResult:(MFMailComposeResult)result error:(NSError*)error 
{
   // make the MFMailComposeViewController disappear
   [self dismissModalViewControllerAnimated:YES];
} // end method mailComposeController:didFinishWithResult:error:

// release this object's memory
- (void)dealloc
{
   [player release]; // release the player AVAudioPlayer
   [files release]; // release the files NSMutableArray
   [super dealloc]; // call the superclass's dealloc method
} // end method dealloc
@end


/**************************************************************************
 * (C) Copyright 2010 by Deitel & Associates, Inc. All Rights Reserved.   *
 *                                                                        *
 * DISCLAIMER: The authors and publisher of this book have used their     *
 * best efforts in preparing the book. These efforts include the          *
 * development, research, and testing of the theories and programs        *
 * to determine their effectiveness. The authors and publisher make       *
 * no warranty of any kind, expressed or implied, with regard to these    *
 * programs or to the documentation contained in these books. The authors *
 * and publisher shall not be liable in any event for incidental or       *
 * consequential damages in connection with, or arising out of, the       *
 * furnishing, performance, or use of these programs.                     *
 *                                                                        *
 * As a user of the book, Deitel & Associates, Inc. grants you the        *
 * nonexclusive right to copy, distribute, display the code, and create   *
 * derivative apps based on the code for noncommercial purposes only--so  *
 * long as you attribute the code to Deitel & Associates, Inc. and        *
 * reference www.deitel.com/books/iPhoneFP/. If you have any questions,   *
 * or specifically would like to use our code for commercial purposes,    *
 * contact deitel@deitel.com.                                             *
 *************************************************************************/

