// Fig. 14.8: VoiceRecorderViewController.m
// VoiceRecorderViewController class implementation.
#import "VoiceRecorderViewController.h"

@implementation VoiceRecorderViewController
@synthesize visualizer; // generate get and set methods for visualizer
@synthesize recordButton; // generate get and set methods for recordButton

// setup the View
- (void)viewDidLoad
{
   [super viewDidLoad]; // call the superclass's viewDidLoad method
   
   // activate the current audio session
   [[AVAudioSession sharedInstance] setActive:YES error:nil];
} // end method viewDidLoad

// called when the user touches the record/stop recording button
- (IBAction)record:sender
{
   // if we’re currently recording
   if (recorder.recording)
   {
      [timer invalidate]; // stop the timer from generating events
      timer = nil; // set time to nil
      [recorder stop]; // stop recording
      
      // set the category of the current audio session
      [[AVAudioSession sharedInstance] setCategory:
         AVAudioSessionCategorySoloAmbient error:nil];
      
      // load the record image
      UIImage *recordImage = [UIImage imageNamed:@"record.png"];
      
      // set the image on the record button
      [recordButton setImage:recordImage forState:UIControlStateNormal];
      
      // create a new NameRecordingViewController
      NameRecordingViewController *controller = 
      [[NameRecordingViewController alloc] init];
      controller.delegate = self; // set controller's delegate to self
      
      // show the NameRecordingViewController
      [self presentModalViewController:controller animated:YES];
   } // end if
   else
   {
      // set the audio session's category to record
      [[AVAudioSession sharedInstance] setCategory:
       AVAudioSessionCategoryRecord error:nil];  
      
      // find the location of the document directory
      NSArray *paths = NSSearchPathForDirectoriesInDomains(
         NSDocumentDirectory, NSUserDomainMask, YES);
      
      // get the first directory
      NSString *dir = [paths objectAtIndex:0];
      
      // create a name for the file using the current system time
      NSString *filename = [NSString stringWithFormat:@"%f.caf", 
                            [[NSDate date] timeIntervalSince1970]];
      
      // create the path using the directory and file name
      NSString *path = [dir stringByAppendingPathComponent:filename];
      
      // create a new NSMutableDictionary for the record settings        
      NSMutableDictionary *settings = [[NSMutableDictionary alloc] init];
      
      // record using the Apple lossless format
      [settings setValue: [NSNumber
         numberWithInt:kAudioFormatAppleLossless] forKey:AVFormatIDKey];
      
      // set the sample rate to 44100 Hz
      [settings setValue:[NSNumber 
                          numberWithFloat:44100.0] forKey:AVSampleRateKey]; 
      
      // set the number of channels for recording
      [settings setValue:[NSNumber numberWithInt:1] 
                  forKey:AVNumberOfChannelsKey];
      
      // set the bit depth
      [settings setValue:[NSNumber numberWithInt:16] 
                  forKey:AVLinearPCMBitDepthKey];
      
      // set whether the format is big endian
      [settings setValue:[NSNumber numberWithBool:NO] 
                  forKey:AVLinearPCMIsBigEndianKey];
      
      // set whether the audio format is floating point
      [settings setValue:[NSNumber numberWithBool:NO] 
                  forKey:AVLinearPCMIsFloatKey];
      [visualizer clear]; // clear the visualizer
      
      [recorder release]; // release the recorder AVAudioRecorder
      
      // initialize recorder with the URL and settings
      recorder =
         [[AVAudioRecorder alloc] initWithURL:[NSURL fileURLWithPath:path]
         settings:settings error:nil];
      [recorder prepareToRecord]; // prepare the recorder to record
      recorder.meteringEnabled = YES; // enable metering for the recorder
      [recorder record]; // start the recording                          
      
      // start a timer
      timer = [NSTimer scheduledTimerWithTimeInterval:0.05 target:self
         selector:@selector(timerFired:) userInfo:nil repeats:YES];
      
      // create the stop recording image
      UIImage *stopImage = [UIImage imageNamed:@"stop.png"];
      
      // change the image on recordButton to the stop image
      [recordButton setImage:stopImage forState:UIControlStateNormal];
   } // end else
} // end method record:

// called when the user finishes picking a name for the recording
- (void)nameRecordingViewController:(NameRecordingViewController *)
controller didGetName:(NSString *)fileName
{
   // append the extension to the chosen name
   fileName = [fileName stringByAppendingPathExtension:@"caf"];
   
   // get the path for the last recorded file
   NSString *path = [recorder.url path];
   
   // get the directory the last file was saved in
   NSString *dir = [path stringByDeletingLastPathComponent];
   
   // append the new file name to the path
   NSString *newPath = [dir stringByAppendingPathComponent:fileName];
   
   // get the default file manager
   NSFileManager *fileManager = [NSFileManager defaultManager];
   
   // rename the old file to the new name the user picked
   [fileManager moveItemAtPath:path toPath:newPath error:nil];
   
   // make the NameRecordingViewController go away
   [self dismissModalViewControllerAnimated:YES];
} // end method nameRecordingViewController:didGetName:

// transition the app to the PlaybackView
- (IBAction)flip:sender
{
   // create a new PlaybackViewController
   PlaybackViewController *playback = 
   [[PlaybackViewController alloc] init];
   
   // set the transition style to fade
   playback.modalTransitionStyle = UIModalTransitionStyleCrossDissolve;
   playback.delegate = self; // set playback's delegate to self
   
   // show the PlaybackViewController
   [self presentModalViewController:playback animated:YES];
   [playback release]; // release the playback PlaybackViewController
} // end method flip:

// delegate method for the PlaybackViewController
- (void)playbackViewControllerDidFinish:
   (PlaybackViewController *)controller
{
   // return to the VoiceRecorderView
   [self dismissModalViewControllerAnimated:YES];
} // end method playbackViewControllerDidFinish:

// called every .05 seconds when the timer generates an event
- (void)timerFired:(NSTimer *)timer
{
   [recorder updateMeters]; // sample the recording to get new data
   
   // set the visualizer's average power level
   [visualizer setPower:[recorder averagePowerForChannel:0]];
   [visualizer setNeedsDisplay]; // redraw the visualizer
} // end method timerFired:

// release this object's memory
- (void)dealloc
{
   [visualizer release]; // release the Visualizer
   [recordButton release]; // release the recordButton UIButton
   [recorder release]; // release the recorder AVAudioRecorder
   [super dealloc]; // call the superclass’s dealloc method
} // end method dealloc
@end // VoiceRecorderViewController class


/**************************************************************************
 * (C) Copyright 2010 by Deitel & Associates, Inc. All Rights Reserved.   *
 *                                                                        *
 * DISCLAIMER: The authors and publisher of this book have used their     *
 * best efforts in preparing the book. These efforts include the          *
 * development, research, and testing of the theories and programs        *
 * to determine their effectiveness. The authors and publisher make       *
 * no warranty of any kind, expressed or implied, with regard to these    *
 * programs or to the documentation contained in these books. The authors *
 * and publisher shall not be liable in any event for incidental or       *
 * consequential damages in connection with, or arising out of, the       *
 * furnishing, performance, or use of these programs.                     *
 *                                                                        *
 * As a user of the book, Deitel & Associates, Inc. grants you the        *
 * nonexclusive right to copy, distribute, display the code, and create   *
 * derivative apps based on the code for noncommercial purposes only--so  *
 * long as you attribute the code to Deitel & Associates, Inc. and        *
 * reference www.deitel.com/books/iPhoneFP/. If you have any questions,   *
 * or specifically would like to use our code for commercial purposes,    *
 * contact deitel@deitel.com.                                             *
 *************************************************************************/

